# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from tqdm import tqdm
import concurrent.futures
import pymol2

from ..functions import *

byres_str = "byres"
within_str = "within"
of_str = "of"
cb_str = "and name CB"

max_cb_dist = 12.0
max_atomid_dist = 5.0


def save_interf(
    coord_path,
    interf_dir=None,
    sym_dist=5.0,
    min_area=200,
    iso_min_simi=0.7,
    use_simpson=True,
    chainid_lst=None,
):

    file_name = get_file_name(coord_path)

    if ".cif" in file_name:
        file_format = "cif"
        return_pdb = False
    elif ".pdb" in file_name:
        file_format = "pdb"
        return_pdb = True

    obj = file_name.replace(f".{file_format}", "")

    if ".gz" in file_name:
        obj = obj.replace(".gz", "")

    with pymol2.PyMOL() as pymol:

        cmd = pymol.cmd

        cmd.load(coord_path, obj)

        cmd.set("dot_solvent", 1)

        if chainid_lst is None:
            chainid_lst = cmd.get_chains(obj)
        else:
            cmd.remove(f"{obj} and not chain {lst_to_str(chainid_lst,join_txt='+')}")

        cmd.split_chains(obj)

        for chainid in chainid_lst:
            chainid_obj = f"{obj}_{chainid}"

            cmd.symexp(f"{chainid_obj}_", chainid_obj, chainid_obj, sym_dist)

        sym_obj_lst = cmd.get_object_list("all")
        sym_obj_lst.remove(obj)

        interf_dict = {coord_path: dict()}

        for chainid in chainid_lst:
            chainid_obj = f"{obj}_{chainid}"

            chainid_area = cmd.get_area(f"{chainid_obj} and polymer")

            interf = 1

            interf_dict[coord_path][chainid] = dict()

            past_cb_lst = list()
            past_atomid_lst = list()

            for sym_obj in sym_obj_lst:
                if chainid_obj != sym_obj:
                    return_chainid = cmd.get_chains(sym_obj)[0]

                    sym_chainid = f"{return_chainid}_{interf}"

                    cmd.alter(sym_obj, f"chain='{sym_chainid}'")

                    interf_obj = f"{chainid_obj}_{interf}"

                    cmd.create(interf_obj, f"{chainid_obj} {sym_obj}")

                    cmd.alter(sym_obj, f"chain='{return_chainid}'")

                    sym_area = cmd.get_area(f"{sym_obj} and polymer")
                    complex_area = cmd.get_area(f"{interf_obj} and polymer")

                    interf_area = ((chainid_area + sym_area) - complex_area) / 2

                    if interf_area >= min_area:

                        chainid_1_sele = f"{interf_obj} and chain {chainid} and polymer"
                        chainid_2_sele = (
                            f"{interf_obj} and chain {sym_chainid} and polymer"
                        )

                        chainid_cb_iterate = lst_to_str(
                            [
                                byres_str,
                                chainid_1_sele,
                                cb_str,
                                within_str,
                                max_cb_dist,
                                of_str,
                                chainid_2_sele,
                                cb_str,
                            ],
                            join_txt=" ",
                        )
                        chainid_atomid_iterate = lst_to_str(
                            [
                                byres_str,
                                chainid_1_sele,
                                within_str,
                                max_atomid_dist,
                                of_str,
                                chainid_2_sele,
                            ],
                            join_txt=" ",
                        )

                        sym_cb_iterate = lst_to_str(
                            [
                                byres_str,
                                chainid_2_sele,
                                cb_str,
                                within_str,
                                max_cb_dist,
                                of_str,
                                chainid_1_sele,
                                cb_str,
                            ],
                            join_txt=" ",
                        )

                        sym_atomid_iterate = lst_to_str(
                            [
                                byres_str,
                                chainid_2_sele,
                                within_str,
                                max_atomid_dist,
                                of_str,
                                chainid_1_sele,
                            ],
                            join_txt=" ",
                        )

                        cont_dict = {
                            "chainid_cb_lst": list(),
                            "chainid_atomid_lst": list(),
                            "sym_cb_lst": list(),
                            "sym_atomid_lst": list(),
                        }

                        cmd.iterate(
                            chainid_cb_iterate,
                            "chainid_cb_lst.append(resi)",
                            space=cont_dict,
                        )
                        cmd.iterate(
                            chainid_atomid_iterate,
                            "chainid_atomid_lst.append(resi)",
                            space=cont_dict,
                        )

                        cmd.iterate(
                            sym_cb_iterate,
                            "sym_cb_lst.append(resi)",
                            space=cont_dict,
                        )
                        cmd.iterate(
                            sym_atomid_iterate,
                            "sym_atomid_lst.append(resi)",
                            space=cont_dict,
                        )

                        chainid_cb_lst = sort_lst(
                            lst_unique(cont_dict["chainid_cb_lst"])
                        )

                        chainid_atomid_lst = sort_lst(
                            lst_unique(cont_dict["chainid_atomid_lst"])
                        )

                        sym_cb_lst = sort_lst(lst_unique(cont_dict["sym_cb_lst"]))
                        sym_atomid_lst = sort_lst(
                            lst_unique(cont_dict["sym_atomid_lst"])
                        )

                        cb_lst = sort_lst(lst_unique(chainid_cb_lst + sym_cb_lst))
                        atomid_lst = sort_lst(
                            lst_unique(chainid_atomid_lst + sym_atomid_lst)
                        )

                        if not (cb_lst in past_cb_lst or atomid_lst in past_atomid_lst):

                            past_cb_lst.append(cb_lst)
                            past_atomid_lst.append(atomid_lst)

                            total_cb_cont = len(chainid_cb_lst)
                            total_atomid_cont = len(chainid_atomid_lst)

                            if (
                                total_cb_cont >= 10 and total_atomid_cont >= 1
                            ) or total_atomid_cont >= 5:

                                interf_path = get_interf_path(
                                    obj,
                                    chainid,
                                    interf,
                                    dir_path=interf_dir,
                                    return_pdb=return_pdb,
                                )
                                cmd.save(interf_path, interf_obj)

                                iso_status = False
                                if use_simpson:
                                    cb_simi = calc_simpson(chainid_cb_lst, sym_cb_lst)
                                    atomid_simi = calc_simpson(
                                        chainid_atomid_lst, sym_atomid_lst
                                    )
                                else:
                                    cb_simi = calc_jaccard(chainid_cb_lst, sym_cb_lst)
                                    atomid_simi = calc_jaccard(
                                        chainid_atomid_lst, sym_atomid_lst
                                    )
                                if (
                                    cb_simi >= iso_min_simi
                                    and atomid_simi >= iso_min_simi
                                ):
                                    iso_status = True

                                interf_dict[coord_path][chainid][interf] = dict()

                                interf_dict[coord_path][chainid][interf][
                                    interf_path_col
                                ] = interf_path
                                interf_dict[coord_path][chainid][interf][
                                    bound_interf_chainid_col
                                ] = sym_chainid
                                interf_dict[coord_path][chainid][interf][
                                    interf_area_col
                                ] = interf_area
                                interf_dict[coord_path][chainid][interf][
                                    cb_cont_col
                                ] = lst_to_str(chainid_cb_lst)
                                interf_dict[coord_path][chainid][interf][
                                    atomid_cont_col
                                ] = lst_to_str(chainid_atomid_lst)
                                interf_dict[coord_path][chainid][interf][
                                    total_cb_cont_col
                                ] = total_cb_cont
                                interf_dict[coord_path][chainid][interf][
                                    total_atomid_cont_col
                                ] = total_atomid_cont
                                interf_dict[coord_path][chainid][interf][
                                    iso_col
                                ] = iso_status

                                interf += 1

    return interf_dict


def prep_interf(
    coord_paths,
    interf_dir=None,
    interf_json_path=None,
    sym_dist=5.0,
    min_area=200,
    iso_min_simi=0.7,
    use_simpson=True,
    chainid_dict=None,
    update_interf=False,
    num_cpu=1,
):

    interf_path = get_dir_path(dir_str=interf_str, dir_path=interf_dir)

    if update_interf:
        delete_path(interf_path)
    append_path(interf_path)

    coord_path_lst = type_lst(coord_paths)

    interf_dict = dict()

    if num_cpu == 1:
        for coord_path in tqdm(
            coord_path_lst,
            desc="Preparing interfaces",
            position=0,
            leave=True,
        ):
            interf_dict = merge_dicts(
                [
                    interf_dict,
                    save_interf(
                        coord_path,
                        interf_dir=interf_dir,
                        sym_dist=sym_dist,
                        min_area=min_area,
                        chainid_lst=chainid_dict[coord_path],
                    ),
                ]
            )
    else:
        with concurrent.futures.ProcessPoolExecutor(max_workers=num_cpu) as executor:
            job_lst = [
                executor.submit(
                    save_interf,
                    coord_path,
                    interf_dir=interf_dir,
                    sym_dist=sym_dist,
                    min_area=min_area,
                    iso_min_simi=iso_min_simi,
                    use_simpson=use_simpson,
                    chainid_lst=chainid_dict[coord_path],
                )
                for coord_path in coord_path_lst
            ]

            for job in tqdm(
                concurrent.futures.as_completed(job_lst),
                desc="Preparing interfaces",
                total=len(job_lst),
                miniters=1,
                position=0,
                leave=True,
            ):

                interf_dict = merge_dicts([interf_dict, job.result()])

    print("Prepared interfaces!")

    if interf_json_path is not None:
        save_json(interf_json_path, interf_dict)
    else:
        return interf_dict



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import streamlit as st

from ..constants import *
from ..functions import *
from ..tools import *

pdb_col_lst = [
    chainid_col,
    bio_lig_col,
]


def pdb_page():

    df = load_st_table(__file__)

    st.sidebar.markdown("## Page Selection")

    pdb_code = st.sidebar.selectbox(
        "Select PDB Entry", [x.upper() for x in lst_col(df, pdb_code_col, unique=True)]
    )
    pdb_df = mask_st_table(df, {pdb_code_col: pdb_code.lower()})

    chainid = st.sidebar.selectbox("Select Chain", lst_col(pdb_df, chainid_col))
    chainid_df = mask_st_table(pdb_df, {chainid_col: chainid})

    st.markdown(
        f"### PDB: [{pdb_code}](https://www.rcsb.org/structure/{pdb_code}) ({df.at[0,gene_class_col]}) - Chain {chainid}"
    )
    st.markdown("---")

    left_col, right_col = st.columns(2)

    left_col.markdown("#### General Information")
    for col in [method_col, resolution_col, r_factor_col, cf_col]:
        left_col.markdown(f"**{rename_col_dict[col]}:** {chainid_df.at[0,col]}")

    right_col.markdown("#### Molecular Annotations")

    annot_df = pd.DataFrame()
    for i, col in enumerate(
        [
            mut_status_col,
            nuc_class_col,
            prot_class_col,
            pharm_class_col,
            match_class_col,
            pocket_class_col,
            interf_class_col,
        ]
    ):
        annot_df.at[i, "Molecular Content"] = rename_col_dict[col]
        annot_df.at[i, "Annotation"] = chainid_df.at[0, col]

    show_st_table(annot_df, st_col=right_col)

    sw1_conf = chainid_df.at[0, sw1_col]
    sw2_conf = chainid_df.at[0, sw2_col]

    sw1_html = f'<p style="font-family:sans-serif; color:{sw1_color}; font-size: large;">{sw1_conf}</p>'
    sw2_html = f'<p style="font-family:sans-serif; color:{sw2_color}; font-size: large;">{sw2_conf}</p>'

    left_col.markdown("#### SW1 Conformation")
    left_col.markdown(sw1_html, unsafe_allow_html=True)
    left_col.markdown("#### SW2 Conformation")
    left_col.markdown(sw2_html, unsafe_allow_html=True)

    st.markdown("---")

    left_view_col, right_view_col = st.columns(2)

    left_view_col.markdown("#### Viewer Settings")

    style_dict = {"Ribbon": "trace", "Cartoon": "oval"}

    cartoon_style = style_dict[
        left_view_col.radio("Backbone Style", ["Ribbon", "Cartoon"])
    ]

    surf_trans = left_view_col.slider(
        "Surface Transparency", min_value=0.0, max_value=1.0
    )

    right_view_col.markdown("#### Bound Ligands")

    check_dict = dict()
    for col in [
        bio_lig_col,
        ion_lig_col,
        pharm_lig_col,
        chem_lig_col,
        mod_lig_col,
        mem_lig_col,
    ]:
        lig = chainid_df.at[0, col]
        if lig != "None":
            check_dict[col] = right_view_col.checkbox(f"{rename_col_dict[col]}: {lig}")

    if len(check_dict.keys()) == 0:
        right_view_col.write("No bound ligands.")

    style_lst = list()
    label_lst = list()

    style_lst.append(
        [
            {
                "chain": chainid,
                "invert": True,
            },
            {"cartoon": {"opacity": 0}},
        ]
    )

    for lig_col, lig_lst in lig_lst_dict.items():
        lig_lst = str_to_lst(chainid_df.at[0, lig_col])
        if "None" not in lig_lst:
            lig_style = "stick"
            lig_color = "whiteCarbon"
            lig_scheme = "colorscheme"
            lig_radius = "0.2"
            if lig_col == ion_lig_col:
                lig_style = "sphere"
                lig_scheme = "color"
                lig_color = "chartreuse"
                lig_radius = 0.8
            for lig in lig_lst:
                style_lst.append(
                    [
                        {
                            "chain": chainid,
                            "resn": [lig],
                        },
                        {lig_style: {lig_scheme: lig_color, "radius": lig_radius}},
                    ]
                )

            if lig_col in list(check_dict.keys()):
                if check_dict[lig_col]:
                    label_lst.append(
                        [
                            lig,
                            {
                                "backgroundColor": "lightgray",
                                "fontColor": "black",
                                "backgroundOpacity": 0.5,
                            },
                            {
                                "chain": chainid,
                                "resn": [lig],
                            },
                        ]
                    )

    pharm_class_lst = str_to_lst(chainid_df.at[0, pharm_class_col])

    if none_pharm_name not in pharm_class_lst:
        if len(pharm_class_lst) > 1:
            pharm_class = other_pharm_name
        else:
            pharm_class = pharm_class_lst[0]

        pharm_color = pharm_color_dict[pharm_class]

        pharm_lig_lst = str_to_lst(chainid_df.at[0, pharm_lig_col])

        for pharm_lig in pharm_lig_lst:
            style_lst.append(
                [
                    {
                        "chain": chainid,
                        "resn": [pharm_lig],
                        "elem": "C",
                    },
                    {"stick": {"color": pharm_color, "radius": 0.2}},
                ]
            )
            style_lst.append(
                [
                    {
                        "chain": chainid,
                        "resn": [pharm_lig],
                        "elem": ["N", "O", "H"],
                    },
                    {"stick": {"colorscheme": "Carbon", "radius": 0.2}},
                ]
            )

            if pharm_class == sp2_name:
                style_lst.append(
                    [
                        {
                            "chain": chainid,
                            "resi": 12,
                        },
                        {"stick": {"colorscheme": "lightgrayCarbon", "radius": 0.2}},
                    ]
                )

    prot_class_lst = str_to_lst(chainid_df.at[0, prot_class_col])

    if none_prot_name not in prot_class_lst:
        if len(prot_class_lst) > 1:
            prot_class = other_prot_name
        else:
            prot_class = prot_class_lst[0]
        prot_color = prot_color_dict[prot_class]

        bound_chainid_lst = str_to_lst(chainid_df.at[0, bound_prot_chainid_col])

        for bound_prot_chainid in bound_chainid_lst:
            style_lst.append(
                [
                    {
                        "chain": bound_prot_chainid,
                    },
                    {
                        "cartoon": {
                            "style": cartoon_style,
                            "color": prot_color,
                            "thickness": 0.2,
                            "opacity": 1,
                        }
                    },
                ]
            )

            prot_label = prot_class
            if len(bound_chainid_lst) > 1:
                prot_label += f" - Chain {bound_prot_chainid}"

            label_lst.append(
                [
                    prot_label,
                    {
                        "backgroundColor": "lightgray",
                        "fontColor": "black",
                        "backgroundOpacity": 0.5,
                    },
                    {"chain": bound_prot_chainid},
                ]
            )

    surface_lst = [
        [
            {"opacity": surf_trans, "color": "white"},
            {"chain": chainid, "hetflag": False},
        ]
    ]

    for loop_name, loop_resids in loop_resid_dict.items():

        if loop_name == sw1_name:
            loop_color = sw1_color
            stick_resid = 32
        elif loop_name == sw2_name:
            loop_color = sw2_color
            stick_resid = 71

        surface_lst.append(
            [
                {"opacity": surf_trans, "color": loop_color},
                {"chain": chainid, "resi": loop_resids, "hetflag": False},
            ]
        )

        style_lst.append(
            [
                {
                    "chain": chainid,
                    "resi": [loop_resids],
                },
                {
                    "cartoon": {
                        "style": cartoon_style,
                        "color": loop_color,
                        "thickness": 0.2,
                    }
                },
            ]
        )

        style_lst.append(
            [
                {
                    "chain": chainid,
                    "resi": [stick_resid],
                    "elem": "C",
                },
                {"stick": {"color": loop_color, "radius": 0.2}},
            ]
        )

        style_lst.append(
            [
                {"chain": chainid, "resi": [stick_resid], "elem": ["O", "N", "H"]},
                {"stick": {"colorscheme": "Carbon", "radius": 0.2}},
            ]
        )

    show_st_structure(
        pdb_code.lower(),
        style_lst=style_lst,
        surface_lst=surface_lst,
        label_lst=label_lst,
        cartoon_style=cartoon_style,
        zoom_dict={"chain": chainid},
    )

    st.markdown("---")
    st.write("Copyright (c) 2022 Mitchell Isaac Parker")



"""input text file will be parsed by comma, space, tab and new line"""


def input_text_file_parser(filename):
    with open(filename) as file:
        file_contents = [line.rstrip() for line in file]
    comma_space_tab_newline_parser = list()
    for n_element in file_contents:
        if "," in n_element:
            n_parsed = n_element.split(",")
            comma_space_tab_newline_parser.append(n_parsed)
        elif " " in n_element:
            n_parsed = n_element.split(" ")
            comma_space_tab_newline_parser.append(n_parsed)
        elif "\t" in n_element:
            n_parsed = n_element.split("\t")
            comma_space_tab_newline_parser.append(n_parsed)
        elif "\n" in n_element:
            n_parsed = n_element.split("\n")
            comma_space_tab_newline_parser.append(n_parsed)
        else:
            comma_space_tab_newline_parser.append(n_element)

    if_list_parse_list = list()
    for n_element in comma_space_tab_newline_parser:
        if type(n_element) == list:
            for d_list_parsed in n_element:
                if d_list_parsed not in if_list_parse_list:
                    if_list_parse_list.append(d_list_parsed)

    final_parsed_list = list()
    for n_elem in if_list_parse_list:
        for _ in range(3):
            n_elem = n_elem.strip(",")
            n_elem = n_elem.strip()

        if len(n_elem) < 4:
            continue
        final_parsed_list.append(n_elem)

    return final_parsed_list



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
import numpy as np
import math
import itertools

from .lst import move_end_lst
from .col import (
    cluster_col,
    pdb_code_col,
    cf_col,
    common_col,
    entropy_col,
    occup_col,
    rama_col,
    rotamer_col,
    bb_seq_col,
    sc_seq_col,
    mean_col,
    max_col,
    nn_dist_col,
    constr_dist_col,
    dist_col_lst,
    interf_area_col,
    pocket_volume_col,
    pocket_score_col,
    iso_col,
)
from .table import (
    build_count_dict,
    build_col_count_dict,
    build_count_table,
    get_col_most_common,
    lst_col,
    lst_by_freq,
    get_val_index_lst,
    mask_equal,
    merge_tables,
    mask_matrix,
)


def is_noise(val):

    noise_status = False
    if val == "Noise":
        noise_status = True
    elif val == -1:
        noise_status = True

    return noise_status


def label_cluster(val, cluster_order):

    if is_noise(val):
        return "Noise"
    else:
        return str(cluster_order.index(val) + 1)


def order_clusters(df, order_col=None):

    temp_df = df.copy(deep=True)
    if order_col is not None:
        temp_df = temp_df.loc[:, [cluster_col, order_col]]
        temp_df = temp_df.drop_duplicates()

    cluster_lst = lst_col(temp_df, cluster_col)

    cluster_order = lst_by_freq(cluster_lst)

    for cluster in cluster_order:
        if is_noise(cluster):
            cluster_order = move_end_lst(cluster_order, "Noise")

    df[cluster_col] = df.apply(
        lambda x: label_cluster(x[cluster_col], cluster_order),
        axis=1,
    )

    return df


def renumber_clusters(df, min_samples, min_pdb=2, min_cf=1, order_col=None):

    count_dict = build_count_dict(lst_col(df, cluster_col, return_str=True))

    check_pdb = False
    if pdb_code_col in list(df.columns):
        pdb_dict = build_col_count_dict(
            df, cluster_col, col_lst=[pdb_code_col], return_str=True
        )
        check_pdb = True

    check_cf = False
    if cf_col in list(df.columns):
        cf_dict = build_col_count_dict(
            df, cluster_col, col_lst=[cf_col], return_str=True
        )
        check_cf = True

    df[cluster_col] = df[cluster_col].map(str)

    label_dict = dict()
    for cluster in lst_col(df, cluster_col, unique=True, return_str=True):

        label = cluster

        if count_dict[cluster] < min_samples:
            label = "Noise"

        if check_pdb:
            if pdb_dict[cluster] < min_pdb:
                label = "Noise"

        if check_cf:
            if cf_dict[cluster] < min_cf:
                label = "Noise"

        label_dict[cluster] = label

    df[cluster_col] = df[cluster_col].map(label_dict)

    df = order_clusters(df, order_col=order_col)

    return df


def get_index_dist_lst(matrix, index_lst, index=None):

    if index is not None:
        index_lst.remove(index)

    return matrix[index_lst, index]


def calc_dist_stat(dist_lst, method="mean", dec=2):

    if len(dist_lst) < 2:
        dist = dist_lst[0]
    else:
        if method == "mean":
            dist = np.mean(dist_lst)
        elif method == "max":
            dist = np.max(dist_lst)
        elif method == "min":
            dist = np.min(dist_lst)
        elif method == "median":
            dist = np.median(dist_lst)
        elif method == "std":
            dist = np.std(dist_lst)

    return round(dist, dec)


def calc_cutoff(dist_lst, dec=2):

    return round(
        calc_dist_stat(dist_lst, method="mean")
        + (2 * calc_dist_stat(dist_lst, method="std")),
        dec,
    )


def build_cutoff_dict(df, matrix, method="mean"):

    cluster_lst = lst_col(df, cluster_col, unique=True, return_str=True)

    cutoff_dict = dict()

    for cluster in cluster_lst:

        index_lst = get_val_index_lst(df, cluster_col, cluster)

        dist_lst = list()

        if len(index_lst) == 1:
            dist_lst.append(0.0)
        else:
            for index in index_lst:
                dist_lst.append(
                    calc_dist_stat(
                        get_index_dist_lst(matrix, index_lst, index=index),
                        method=method,
                    )
                )

        cutoff_dict[cluster] = calc_cutoff(dist_lst)

    return cutoff_dict


def apply_dist_constr(
    df,
    dist_col,
    matrix,
    min_samples=None,
    max_dist=None,
    constr_method=None,
    order_col=None,
):
    df[cluster_col] = df[cluster_col].map(str)

    df_index_lst = list(df.index.values)

    if max_dist is None:
        cutoff_dict = build_cutoff_dict(df, matrix, method=constr_method)

    pre_count_dict = build_col_count_dict(df, cluster_col)

    calc_dist = True
    while calc_dist:

        calc_dist = False

        for index in df_index_lst:

            cluster = df.at[index, cluster_col]

            index_lst = get_val_index_lst(df, cluster_col, cluster)

            if min_samples is not None:
                if not is_noise(cluster):
                    if len(index_lst) < min_samples:
                        cluster = "Noise"
                        df.at[index, cluster_col] = cluster
                        calc_dist = True

            if len(index_lst) == 1:
                dist = 0.0
            else:
                dist = calc_dist_stat(
                    get_index_dist_lst(matrix, index_lst, index=index),
                    method=constr_method,
                )

            df.at[index, dist_col] = dist

            if not is_noise(cluster):

                prune_cluster = False

                dist_cutoff = max_dist
                if max_dist is None:
                    dist_cutoff = cutoff_dict[cluster]

                if dist > dist_cutoff:
                    prune_cluster = True

                if prune_cluster:
                    df.at[index, cluster_col] = "Noise"
                    calc_dist = True

    post_count_dict = build_col_count_dict(df, cluster_col)

    pruned = 0
    for cluster in list(post_count_dict.keys()):
        if cluster != "Noise":
            pruned += pre_count_dict[cluster] - post_count_dict[cluster]

    df = order_clusters(df, order_col=order_col)

    return df, pruned


def prune_cluster_members(
    df,
    min_samples,
    matrix,
    max_nn_dist=None,
    constr_matrix=None,
    max_constr_dist=None,
    constr_method="mean",
    min_pdb=3,
    min_cf=1,
    order_col=None,
):

    df = renumber_clusters(
        df, min_samples, min_pdb=min_pdb, min_cf=min_cf, order_col=order_col
    )

    df, nn_pruned = apply_dist_constr(
        df,
        nn_dist_col,
        matrix,
        min_samples=min_samples,
        max_dist=max_nn_dist,
        constr_method="min",
        order_col=order_col,
    )

    if constr_matrix is not None:
        if max_constr_dist is None:
            max_constr_dist = build_cutoff_dict(df, constr_matrix, method=constr_method)

        df, constr_pruned = apply_dist_constr(
            df,
            constr_dist_col,
            constr_matrix,
            min_samples=min_samples,
            max_dist=max_constr_dist,
            constr_method=constr_method,
            order_col=order_col,
        )

    df = renumber_clusters(
        df,
        min_samples,
        min_pdb=min_pdb,
        min_cf=min_cf,
        order_col=order_col,
    )

    if constr_matrix is not None:
        return (df, nn_pruned, constr_pruned)
    else:
        return (df, nn_pruned)


def merge_clusters(df, matrix, max_merge_dist, merge_method="mean"):

    cluster_lst = [x for x in lst_col(df, cluster_col, unique=True) if not is_noise(x)]

    cluster_pairs = itertools.combinations(cluster_lst, 2)

    merge_dict = dict()

    for cluster_1, cluster_2 in cluster_pairs:

        index_1_lst = get_val_index_lst(df, cluster_col, cluster_1)
        index_2_lst = get_val_index_lst(df, cluster_col, cluster_2)

        if (
            calc_dist_stat(
                mask_matrix(matrix, index_1_lst, index_2_lst), method=merge_method
            )
            <= max_merge_dist
        ):
            merge_cluster = cluster_lst[
                min(cluster_lst.index(cluster_1), cluster_lst.index(cluster_2))
            ]
            merge_dict[cluster_1] = merge_cluster
            merge_dict[cluster_2] = merge_cluster

    df[cluster_col] = df[cluster_col].replace(merge_dict)

    return df


def calc_entropy(vals, dec=2):

    return round(
        (
            float(-1)
            * float(np.max(vals))
            / float(np.sum(vals))
            * (math.log10(float(np.max(vals)) / float(np.sum(vals))))
        ),
        dec,
    )


def calc_occupancy(vals, dec=2):

    return round(float(np.max(vals)) / float(np.sum(vals)), dec)


def add_dih_stats(temp_df, sum_df, col, index):

    count_dict = build_col_count_dict(temp_df, col)

    common = [x for x in get_col_most_common(temp_df, col) if "-" not in x][0]
    entropy = calc_entropy(list(count_dict.values()))
    occupancy = calc_occupancy(list(count_dict.values()))

    sum_df.at[index, f"{common_col}_{col}"] = common
    sum_df.at[index, f"{entropy_col}_{col}"] = entropy
    sum_df.at[index, f"{occup_col}_{col}"] = occupancy

    return sum_df


def dist_to_dih(dist, dec=2):

    return round(math.degrees(math.acos(1 - (float(dist) / 2))), dec)


def build_sum_table(df):

    df_col_lst = list(df.columns)

    dih_dist = False
    if rama_col in df_col_lst or rotamer_col in df_col_lst:
        dih_dist = True

    add_col_lst = [rama_col, rotamer_col, bb_seq_col, sc_seq_col, iso_col]

    df[cluster_col] = df[cluster_col].map(str)

    cluster_lst = lst_col(df, cluster_col, unique=True, return_str=True)

    sum_df = pd.DataFrame()

    stat_col_lst = (
        dist_col_lst + [interf_area_col] + [pocket_volume_col] + [pocket_score_col]
    )

    for cluster in cluster_lst:

        temp_df = mask_equal(df, cluster_col, cluster)

        index = cluster_lst.index(cluster)

        sum_df.at[index, cluster_col] = cluster

        for col in stat_col_lst:
            if col in df_col_lst:
                dist_lst = lst_col(temp_df, col, return_float=True)

                mean_dist = calc_dist_stat(dist_lst, method="mean")
                max_dist = calc_dist_stat(dist_lst, method="max")

                val_lst = [mean_dist, max_dist]

                if dih_dist:
                    if col == nn_dist_col:
                        val_lst = [dist_to_dih(x) for x in val_lst]

                sum_df.at[index, f"{col}_{mean_col}"] = val_lst[0]
                sum_df.at[index, f"{col}_{max_col}"] = val_lst[1]

        if "Noise" not in cluster:
            for add_col in add_col_lst:
                if add_col in df_col_lst:
                    sum_df = add_dih_stats(temp_df, sum_df, add_col, index)

    sum_df = merge_tables(sum_df, build_count_table(df, cluster_col))

    sum_df = sum_df.fillna("-")

    return sum_df


from src.download.modules import *
from src.download.lookfilesinside import look_what_is_inside
from src.renum.mmCIF.new_mmCIF import master_mmCIF_renumber_function


REMARK_mmCIF = ["#\n",
                "loop_\n",
                "_database_PDB_remark.id       1\n",
                "_database_PDB_remark.text\n",
                ";File processed by PDBrenum: http://dunbrack3.fccc.edu/PDBrenum\n",
                "Author sequence numbering is replaced with UniProt numbering according to\n",
                "alignment by SIFTS (https://www.ebi.ac.uk/pdbe/docs/sifts/).\n",
                "Only chains with UniProt sequences in SIFTS are renumbered.\n",
                "Residues in UniProt chains without UniProt residue numbers in SIFTS\n",
                "(e.g., sequence tags) are given residue numbers 50000+label_seq_id\n",
                "(where label_seq_id is the 1-to-N residue numbering of each chain.\n",
                "Ligands are numbered 50000+their residue number in the original file.\n",
                "The _poly_seq_scheme table contains a correspondence between the\n",
                "1-to-N sequence (seq_id), the new numbering based on UniProt (pdb_seq_num =\n",
                "auth_seq_id in the _atom_site records), and the author numbering\n",
                "in the original mmCIF file from the PDB (auth_seq_num).\n",
                ";\n",
                "#\n"]


def check_assemblies(mmCIF_assembly, default_output_path_to_mmCIF_assembly):
    output_mmCIF_assembly_files_were_found_list = list()
    output_mmCIF_assembly_files_were_found_list.append(mmCIF_assembly)
    new_order_with_remark = 0

    for name in output_mmCIF_assembly_files_were_found_list:
        not_gzip = 1
        try:
            list_of_lines_from_assembly_file = gzip.open(
                Path(str(default_output_path_to_mmCIF_assembly) + "/" + name), 'rt').readlines()
        except OSError:
            # maybe not archived
            try:
                list_of_lines_from_assembly_file = open(
                    Path(str(default_output_path_to_mmCIF_assembly) + "/" + name), 'rt').readlines()
                not_gzip = 0
            except Exception:
                # broken archive
                os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))
                continue
        except Exception:
            # broken archive
            os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))
            continue

        # check if file startswith "_atom_site" table at the beginning
        try:
            if "_atom_site" in list_of_lines_from_assembly_file[3] and "loop_" in list_of_lines_from_assembly_file[2]:
                pass
            else:
                if "renum" in name:
                    for line in list_of_lines_from_assembly_file:
                        if line.startswith("_entry.id"):
                            new_order_with_remark = (list_of_lines_from_assembly_file[:(list_of_lines_from_assembly_file.index(line)) + 1]
                                                     + REMARK_mmCIF
                                                     + list_of_lines_from_assembly_file[(list_of_lines_from_assembly_file.index(line)) + 2:])
                    if new_order_with_remark == 0:
                        new_order_with_remark = list_of_lines_from_assembly_file
                        print(name)

                    if not_gzip != 0:
                        with gzip.open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name), "wt") as gzip_out:
                            for listitem in new_order_with_remark:
                                gzip_out.write(listitem)

                    else:
                        with open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name), "wt") as file_out:
                            for listitem in new_order_with_remark:
                                file_out.write(listitem)

                else:
                    new_order_with_remark = list_of_lines_from_assembly_file
                    if not_gzip != 0:
                        with gzip.open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name.split(".")[0] + "_renum.cif.gz"),
                                       "wt") as gzip_out:
                            for listitem in new_order_with_remark:
                                gzip_out.write(listitem)
                        os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))
                    else:
                        with open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name.split(".")[0] + "_renum.cif"), "wt") as file_out:
                            for listitem in new_order_with_remark:
                                file_out.write(listitem)
                        os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))

                return name

        except IndexError:
            # empty file
            os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))
            continue

        try:
            new_order_for_assembly_file = (list_of_lines_from_assembly_file[:1]
                                           + list_of_lines_from_assembly_file[list_of_lines_from_assembly_file.index("#\n", 2):]
                                           + list_of_lines_from_assembly_file[2:list_of_lines_from_assembly_file.index("#\n", 2)]
                                           + ["#\n"])

            if "renum" in name:
                for line in new_order_for_assembly_file:
                    if line.startswith("_entry.id"):
                        new_order_with_remark = (new_order_for_assembly_file[:(new_order_for_assembly_file.index(line)) + 1]
                                                 + REMARK_mmCIF
                                                 + new_order_for_assembly_file[(new_order_for_assembly_file.index(line)) + 2:])

                if new_order_with_remark != 0:
                    if not_gzip != 0:
                        with gzip.open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name), "wt") as gzip_out:
                            for listitem in new_order_with_remark:
                                gzip_out.write(listitem)

                    else:
                        with open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name), "wt") as file_out:
                            for listitem in new_order_with_remark:
                                file_out.write(listitem)
            else:
                new_order_with_remark = new_order_for_assembly_file
                if not_gzip != 0:
                    with gzip.open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name.split(".")[0] + "_renum.cif.gz"), "wt") as gzip_out:
                        for listitem in new_order_with_remark:
                            gzip_out.write(listitem)
                    os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))
                else:
                    with open(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name.split(".")[0] + "_renum.cif"), "wt") as file_out:
                        for listitem in new_order_with_remark:
                            file_out.write(listitem)
                    os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))
            return name

        except ValueError:
            # file isn't complete
            os.remove(Path(str(default_output_path_to_mmCIF_assembly) + "/" + name))


def ProcessPool_run_renum_mmCIF(format_mmCIF, mmCIF_to_renumber, default_input_path_to_mmCIF,
                                default_input_path_to_SIFTS, default_output_path_to_mmCIF, default_mmCIF_num,
                                gzip_mode, exception_AccessionIDs, nproc):
    first_res = 0

    for i in range(3):
        if not os.path.exists(default_output_path_to_mmCIF):
            os.makedirs(default_output_path_to_mmCIF)

        # renumber loop
        resulting = list()
        executor = ProcessPoolExecutor(max_workers=nproc)
        partial_master_mmCIF_renumber_function = partial(master_mmCIF_renumber_function,
                                                         default_input_path_to_mmCIF=default_input_path_to_mmCIF,
                                                         default_input_path_to_SIFTS=default_input_path_to_SIFTS,
                                                         default_output_path_to_mmCIF=default_output_path_to_mmCIF,
                                                         default_mmCIF_num=default_mmCIF_num, gzip_mode=gzip_mode,
                                                         exception_AccessionIDs=exception_AccessionIDs)
        jobs = [executor.submit(partial_master_mmCIF_renumber_function, mmCIF_files) for mmCIF_files in mmCIF_to_renumber]
        for job in tqdm.tqdm(as_completed(jobs), total=len(jobs), miniters=1, position=0,
                             leave=True, desc="Renumbering " + format_mmCIF + " files"):
            result = job.result()
            if result is None:
                continue
            resulting.append(result)

        if i == 0:
            first_res = resulting

        if format_mmCIF == "mmCIF_assembly":
            output_mmCIF = look_what_is_inside('output_mmCIF_assembly', default_output_path_to_mmCIF_assembly=default_output_path_to_mmCIF)
        else:
            output_mmCIF = look_what_is_inside('output_mmCIF', default_output_path_to_mmCIF=default_output_path_to_mmCIF)

        # checker loop
        check_list = list()
        executor = ProcessPoolExecutor(max_workers=nproc)
        partial_reform_assembly = partial(check_assemblies, default_output_path_to_mmCIF_assembly=default_output_path_to_mmCIF)
        jobs = [executor.submit(partial_reform_assembly, assembly_files) for assembly_files in output_mmCIF]
        for job in tqdm.tqdm(as_completed(jobs), total=len(jobs), miniters=1, position=0,
                             leave=True, desc="Checking " + format_mmCIF + " files"):
            resultus = job.result()
            check_list.append(resultus)

        if format_mmCIF == "mmCIF_assembly":
            output_mmCIF = look_what_is_inside('output_mmCIF_assembly', default_output_path_to_mmCIF_assembly=default_output_path_to_mmCIF)
        else:
            output_mmCIF = look_what_is_inside('output_mmCIF', default_output_path_to_mmCIF=default_output_path_to_mmCIF)

        output_mmCIF_4char = set()
        for n in output_mmCIF:
            output_mmCIF_4char.add(n[:4])

        if len(check_list) <= len(output_mmCIF):
            break
        else:
            new_round_mmCIF_to_renumber = set()
            for n in mmCIF_to_renumber:
                if n[:4] in output_mmCIF_4char:
                    continue
                else:
                    new_round_mmCIF_to_renumber.add(n)
            mmCIF_to_renumber = new_round_mmCIF_to_renumber

    return first_res



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

from ..functions import *

rbd_pfam = "RBD"
pi3k_pfam = "PI3K_RBD"
ra_pfam = "RA"
ra_gef_pfam = "RA,RasGEF"
gef_pfam = "RasGEF"
gap_pfam = "RasGAP"

effector_name = "Effector"
gef_name = "GEF"
gap_name = "GAP"
binder_name = "Binder"
other_prot_name = "Other"
none_prot_name = "None"

gef_cdc_name = "CDC25"
gef_rem_name = "REM"

prot_class_dict = {
    effector_name: [rbd_pfam, pi3k_pfam, ra_pfam, ra_gef_pfam],
    gef_name: [gef_pfam],
    gap_name: [gap_pfam],
    binder_name: [binder_name],
    other_prot_name: [other_prot_name],
    none_prot_name: [none_prot_name],
}

prot_class_lst = [
    effector_name,
    gap_name,
    f"{gef_name}.{gef_cdc_name}",
    f"{gef_name}.{gef_rem_name}",
    binder_name,
    other_prot_name,
    none_prot_name,
]

prot_color_dict = get_lst_colors(
    [
        effector_name,
        gap_name,
        gef_name,
        binder_name,
        other_prot_name,
        none_prot_name,
    ],
    palette="Set2",
    return_dict=True,
)

prot_color_dict[f"{gef_name}.{gef_cdc_name}"] = prot_color_dict[gef_name]
prot_color_dict[f"{gef_name}.{gef_rem_name}"] = prot_color_dict[gef_name]


prot_pfam_dict = {
    "RASF1_HUMAN": "RA",
    "RADIL_HUMAN": "RA",
    "RAPH1_HUMAN": "RA",
    "RASF1_CAEEL": "RA",
    "SPRI_DROME": "RA",
    "RASF5_HUMAN": "RA",
    "RPGF2_MOUSE": "RA,RasGEF",
    "RPGF6_HUMAN": "RA,RasGEF",
    "RASF5_MOUSE": "RA",
    "RPGF2_BOVIN": "RA,RasGEF",
    "SNX27_MOUSE": "RA",
    "PLCE1_HUMAN": "RA,RasGEF",
    "PLCE1_MOUSE": "RA,RasGEF",
    "SNX27_HUMAN": "RA",
    "MYO9B_MOUSE": "RA",
    "SNX27_RAT": "RA",
    "RASF2_MOUSE": "RA",
    "RAIN_HUMAN": "RA",
    "RPGF2_RAT": "RA,RasGEF",
    "MYO10_BOVIN": "RA",
    "MYO9B_HUMAN": "RA",
    "MY9AA_DANRE": "RA",
    "RAIN_MOUSE": "RA",
    "RIN2_HUMAN": "RA",
    "RPGF2_CANLF": "RA,RasGEF",
    "RASF2_HUMAN": "RA",
    "MST50_MAGO7": "RA",
    "PKHH3_HUMAN": "RA",
    "STE50_YEAST": "RA",
    "RASF4_HUMAN": "RA",
    "RADIL_DANRE": "RA",
    "RHG20_RAT": "RA",
    "STE4_SCHPO": "RA",
    "RASF4_RAT": "RA",
    "RASF1_MOUSE": "RA",
    "RGL2_MOUSE": "RA,RasGEF",
    "SNX27_BOVIN": "RA",
    "RGL2_HUMAN": "RA,RasGEF",
    "RASF5_RAT": "RA",
    "RASF7_MOUSE": "RA",
    "RPGF_CAEEL": "RA,RasGEF",
    "RASF6_HUMAN": "RA",
    "RGL1_DANRE": "RA,RasGEF",
    "RHG20_HUMAN": "RA",
    "RGL3_MOUSE": "RA,RasGEF",
    "PKHH3_RAT": "RA",
    "RASF2_RAT": "RA",
    "RASF6_MOUSE": "RA",
    "RGL1_HUMAN": "RA,RasGEF",
    "RGL3_HUMAN": "RA,RasGEF",
    "Y7875_DICDI": "RA",
    "RASF4_MOUSE": "RA",
    "RASF8_HUMAN": "RA",
    "RASF6_RAT": "RA",
    "RASF7_HUMAN": "RA",
    "PLCE1_RAT": "RA,RasGEF",
    "RHG20_MOUSE": "RA",
    "RGL2_CANLF": "RA,RasGEF",
    "MYO9A_RAT": "RA",
    "MYO9B_RAT": "RA",
    "RIN1_HUMAN": "RA",
    "PLCE1_CAEEL": "RA,RasGEF",
    "RGL1_MOUSE": "RA,RasGEF",
    "RPGF2_HUMAN": "RA,RasGEF",
    "RADIL_MOUSE": "RA",
    "RIN2_MOUSE": "RA",
    "RASF3_MOUSE": "RA",
    "AB1IP_XENLA": "RA",
    "GNDS_RAT": "RA,RasGEF",
    "GRB7_RAT": "RA",
    "AB1IP_MOUSE": "RA",
    "AB1IP_HUMAN": "RA",
    "AFAD_MOUSE": "RA",
    "DGKQ_MOUSE": "RA",
    "AB1IP_CHICK": "RA",
    "AFAD_HUMAN": "RA",
    "DGKQ_HUMAN": "RA",
    "ARAP1_HUMAN": "RA",
    "ARAP2_HUMAN": "RA",
    "ARAP1_MOUSE": "RA",
    "GRB10_PIG": "RA",
    "ARAP3_MOUSE": "RA",
    "AB1IP_DANRE": "RA",
    "GNDS_HUMAN": "RA,RasGEF",
    "ARAP3_HUMAN": "RA",
    "GRB14_HUMAN": "RA",
    "AFAD_RAT": "RA",
    "DGKQ_RAT": "RA",
    "ARAP2_MOUSE": "RA",
    "GRB7_HUMAN": "RA",
    "GRB10_MOUSE": "RA",
    "GRB14_BOVIN": "RA",
    "MYO9A_MOUSE": "RA",
    "GRB14_MOUSE": "RA",
    "GRB14_RAT": "RA",
    "GRB10_HUMAN": "RA",
    "MYO9A_HUMAN": "RA",
    "GNDS_MOUSE": "RA,RasGEF",
    "MYO10_HUMAN": "RA",
    "GRB7_MOUSE": "RA",
    "MY9AB_DANRE": "RA",
    "GRB7_BOVIN": "RA",
    "GRB10_RAT": "RA",
    "RASF3_HUMAN": "RA",
    "PKHH3_MOUSE": "RA",
    "RASF8_MOUSE": "RA",
    "RAF1_HUMAN": "RBD",
    "RAF1_MOUSE": "RBD",
    "TIAM1_MOUSE": "RBD",
    "RAF1_CHICK": "RBD",
    "RGS_DROME": "RBD",
    "RAF1_PONAB": "RBD",
    "RGS14_HUMAN": "RBD",
    "RGS14_MOUSE": "RBD",
    "RAF1_RAT": "RBD",
    "RAF1_BOVIN": "RBD",
    "RGS12_RAT": "RBD",
    "TIAM1_HUMAN": "RBD",
    "RGS14_RAT": "RBD",
    "RGS12_HUMAN": "RBD",
    "ARAF_HUMAN": "RBD",
    "ARAF_RAT": "RBD",
    "BRAF_COTJA": "RBD",
    "BRAF_HUMAN": "RBD",
    "BRAF_MOUSE": "RBD",
    "KRAF1_CAEEL": "RBD",
    "KRAF1_DROME": "RBD",
    "RIP3_DICDI": "RBD",
    "BRAF_CHICK": "RBD",
    "SIF1_DROME": "RBD",
    "ARAF_MOUSE": "RBD",
    "SIF2_DROME": "RBD",
    "RGS12_MOUSE": "RBD",
    "RAF1_XENLA": "RBD",
    "ARAF_PIG": "RBD",
    "KRAF1_CAEBR": "RBD",
    "PK3CD_HUMAN": "PI3K_RBD",
    "PK3CD_MOUSE": "PI3K_RBD",
    "PI3K2_DICDI": "PI3K_RBD",
    "P3C2A_HUMAN": "PI3K_RBD",
    "PK3CG_PIG": "PI3K_RBD",
    "PK3CA_BOVIN": "PI3K_RBD",
    "P3C2B_HUMAN": "PI3K_RBD",
    "PK3CG_MOUSE": "PI3K_RBD",
    "PK3CA_RAT": "PI3K_RBD",
    "PK3CB_HUMAN": "PI3K_RBD",
    "P3C2A_MOUSE": "PI3K_RBD",
    "PI3K1_DICDI": "PI3K_RBD",
    "AGE1_CAEEL": "PI3K_RBD",
    "PK3CB_MOUSE": "PI3K_RBD",
    "PK3CA_HUMAN": "PI3K_RBD",
    "PK3CG_HUMAN": "PI3K_RBD",
    "PK3CB_RAT": "PI3K_RBD",
    "PK3CA_MOUSE": "PI3K_RBD",
    "P3C2A_PONAB": "PI3K_RBD",
    "P3C2G_HUMAN": "PI3K_RBD",
    "P3C2G_RAT": "PI3K_RBD",
    "PI3K3_DICDI": "PI3K_RBD",
    "AGE1_CAEBR": "PI3K_RBD",
    "SOS1_MOUSE": "RasGEF",
    "SH2D3_HUMAN": "RasGEF",
    "RGRF1_MOUSE": "RasGEF",
    "LTE1_YEAST": "RasGEF",
    "RPGF1_HUMAN": "RasGEF",
    "SOS_DROME": "RasGEF",
    "RPGF3_RAT": "RasGEF",
    "SH2D3_MOUSE": "RasGEF",
    "RGRF2_MOUSE": "RasGEF",
    "RPGF3_MOUSE": "RasGEF",
    "RPGF4_RAT": "RasGEF",
    "SOS2_HUMAN": "RasGEF",
    "RPGF4_HUMAN": "RasGEF",
    "RGRF1_RAT": "RasGEF",
    "RGPS1_HUMAN": "RasGEF",
    "RPGF3_HUMAN": "RasGEF",
    "RGPS1_DANRE": "RasGEF",
    "SOS_CAEEL": "RasGEF",
    "RG1BA_DANRE": "RasGEF",
    "RPGF5_RAT": "RasGEF",
    "SDC25_YEAS8": "RasGEF",
    "GRP2A_XENLA": "RasGEF",
    "RPGFL_MOUSE": "RasGEF",
    "RPGF1_CAEEL": "RasGEF",
    "RGPS1_CHICK": "RasGEF",
    "RGPS2_MACFA": "RasGEF",
    "RGRF2_RAT": "RasGEF",
    "RGPS2_HUMAN": "RasGEF",
    "STE6_SCHPO": "RasGEF",
    "RPGFL_PONPY": "RasGEF",
    "SDC25_YEAS6": "RasGEF",
    "SDC25_YEAS1": "RasGEF",
    "RGF1C_MOUSE": "RasGEF",
    "GRP2B_XENLA": "RasGEF",
    "KNDC1_HUMAN": "RasGEF",
    "GRP4_BOVIN": "RasGEF",
    "RGF1C_MACFA": "RasGEF",
    "RPGFL_HUMAN": "RasGEF",
    "RGF1C_HUMAN": "RasGEF",
    "RG1BB_DANRE": "RasGEF",
    "RGRF2_DANRE": "RasGEF",
    "RGPS1_MOUSE": "RasGEF",
    "RPGF5_MOUSE": "RasGEF",
    "RPGF5_HUMAN": "RasGEF",
    "YL016_YEAST": "RasGEF",
    "RGPS2_MOUSE": "RasGEF",
    "RPGF4_MOUSE": "RasGEF",
    "RGRF2_HUMAN": "RasGEF",
    "SDC25_YEAS7": "RasGEF",
    "SOS2_MOUSE": "RasGEF",
    "SOS1_HUMAN": "RasGEF",
    "SDC25_YEASX": "RasGEF",
    "RGRF1_HUMAN": "RasGEF",
    "BCAR3_MOUSE": "RasGEF",
    "EFC25_SCHPO": "RasGEF",
    "GBPC_DICDI": "RasGEF",
    "GRP2_MOUSE": "RasGEF",
    "BCAR3_HUMAN": "RasGEF",
    "CDC25_CANAL": "RasGEF",
    "CDC25_LACKL": "RasGEF",
    "CDC25_YEAST": "RasGEF",
    "BCAR3_RAT": "RasGEF",
    "BCAR3_BOVIN": "RasGEF",
    "BEM2_ASHGO": "RasGEF",
    "BUD5_YEAST": "RasGEF",
    "GFLB_DICDI": "RasGEF",
    "GRP3_HUMAN": "RasGEF",
    "GEFE_DICDI": "RasGEF",
    "GEFG_DICDI": "RasGEF",
    "GEFH_DICDI": "RasGEF",
    "GEFK_DICDI": "RasGEF",
    "GEFN_DICDI": "RasGEF",
    "GEFO_DICDI": "RasGEF",
    "GEFX_DICDI": "RasGEF",
    "GEFY_DICDI": "RasGEF",
    "GEFA_DICDI": "RasGEF",
    "GEFB_DICDI": "RasGEF",
    "GEFC_DICDI": "RasGEF",
    "GRP2_BOVIN": "RasGEF",
    "BEM2_YEAST": "RasGEF",
    "GEFD_DICDI": "RasGEF",
    "GEFF_DICDI": "RasGEF",
    "GEFI_DICDI": "RasGEF",
    "GEFJ_DICDI": "RasGEF",
    "GEFL_DICDI": "RasGEF",
    "GEFM_DICDI": "RasGEF",
    "GEFP_DICDI": "RasGEF",
    "GEFQ_DICDI": "RasGEF",
    "GEFR_DICDI": "RasGEF",
    "GEFS_DICDI": "RasGEF",
    "GEFV_DICDI": "RasGEF",
    "GEFW_DICDI": "RasGEF",
    "LTE1_ASHGO": "RasGEF",
    "GRP4_RAT": "RasGEF",
    "GRP1_XENLA": "RasGEF",
    "GRP4_HUMAN": "RasGEF",
    "GRP1_HUMAN": "RasGEF",
    "BCAR3_XENLA": "RasGEF",
    "BCAR3_MACFA": "RasGEF",
    "GBPD_DICDI": "RasGEF",
    "GRP4_MOUSE": "RasGEF",
    "GRP1_XENTR": "RasGEF",
    "KNDC1_MOUSE": "RasGEF",
    "GRP1_RAT": "RasGEF",
    "LTE1_CANGA": "RasGEF",
    "LTE1_KLULA": "RasGEF",
    "GRP1_MOUSE": "RasGEF",
    "GRP2_RAT": "RasGEF",
    "GRP2_HUMAN": "RasGEF",
    "RGDSR_HUMAN": "RasGEF",
    "RGF1B_HUMAN": "RasGEF",
    "RGF1B_PONAB": "RasGEF",
    "RGF1A_HUMAN": "RasGEF",
    "RGF1A_XENTR": "RasGEF",
    "RGF1B_BOVIN": "RasGEF",
    "RGF1B_MOUSE": "RasGEF",
    "RGF1B_XENTR": "RasGEF",
    "C3G_DROME": "RasGEF",
    "RASA3_HUMAN": "RasGAP",
    "RASL3_HUMAN": "RasGAP",
    "RNG2_SCHPO": "RasGAP",
    "RASL2_HUMAN": "RasGAP",
    "NF1_HUMAN": "RasGAP",
    "IQGA1_MOUSE": "RasGAP",
    "IQG1_YEAST": "RasGAP",
    "RGAA_DICDI": "RasGAP",
    "IQGA2_HUMAN": "RasGAP",
    "RASA1_HUMAN": "RasGAP",
    "SYGP1_MOUSE": "RasGAP",
    "SYGP1_RAT": "RasGAP",
    "RASA1_RAT": "RasGAP",
    "RME6_CAEEL": "RasGAP",
    "BUD2_CANAL": "RasGAP",
    "DAB2P_HUMAN": "RasGAP",
    "GAPD1_MOUSE": "RasGAP",
    "GAP1_DROME": "RasGAP",
    "DAB2P_RAT": "RasGAP",
    "GAPD1_HUMAN": "RasGAP",
    "GAPA_DICDI": "RasGAP",
    "NF1_RAT": "RasGAP",
    "IQGA3_HUMAN": "RasGAP",
    "NF1_MOUSE": "RasGAP",
    "IQGA1_HUMAN": "RasGAP",
    "SYGP1_HUMAN": "RasGAP",
    "RASA2_HUMAN": "RasGAP",
    "RASA3_RAT": "RasGAP",
    "GAP2_DROME": "RasGAP",
    "RASA3_BOVIN": "RasGAP",
    "RASL3_MOUSE": "RasGAP",
    "RAS4B_HUMAN": "RasGAP",
    "RASL3_BOVIN": "RasGAP",
    "RME6_DROME": "RasGAP",
    "RASA1_BOVIN": "RasGAP",
    "RME6_CAEBR": "RasGAP",
    "RME6_DROPS": "RasGAP",
    "RASA3_MOUSE": "RasGAP",
    "NGAP_HUMAN": "RasGAP",
    "NFAA_DICDI": "RasGAP",
    "RASL1_MOUSE": "RasGAP",
    "RASL1_HUMAN": "RasGAP",
    "RASL2_MOUSE": "RasGAP",
    "RASA2_MOUSE": "RasGAP",
    "RASA2_RAT": "RasGAP",
    "IRA1_YEAST": "RasGAP",
    "GAPD1_XENLA": "RasGAP",
    "BUD2_YEAST": "RasGAP",
    "GAP1_SCHPO": "RasGAP",
    "GAP1_CAEEL": "RasGAP",
    "GAP2_CAEEL": "RasGAP",
    "GAPD1_BOVIN": "RasGAP",
    "IQG1_CANAL": "RasGAP",
    "NGAP_DICDI": "RasGAP",
    "IRA2_YEAST": "RasGAP",
    "IQGA2_MOUSE": "RasGAP",
    "DAB2P_MOUSE": "RasGAP",
}


# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import os
import shutil
import gzip
import json
import pandas as pd
import numpy as np

from .table import get_col_order, order_cols, order_rows

rascore_str = "rascore"
build_str = "build"
classify_str = "classify"
cluster_str = "cluster"

scripts_str = "scripts"
data_str = "data"
pages_str = "pages"

pdbaa_str = "pdbaa"
core_str = "core"
rcsb_str = "rcsb"
renum_str = "renum"
rcsb_assembly_str = "rcsb_assembly"
renum_assembly_str = "renum_assembly"
sifts_str = "sifts"
edia_str = "edia"
seq_str = "sequence"
interf_str = "interface"
pocket_str = "pocket"
lig_str = "ligand"
eds_str = "eds"
map_str = "2mFo-DFc"
diff_str = "mFo-DFc"
sup_str = "super"


def path_exists(path):

    exists = False
    if path is not None:
        if os.path.isfile(path):
            exists = True
        elif os.path.isdir(path):
            exists = True

    return exists


def append_path(path):

    if not path_exists(path):
        os.makedirs(path)


def get_dir_name(dir_path):

    if "/" in dir_path:
        dir_name = dir_path.rsplit("/", 1)[0]
    else:
        dir_name = os.getcwd()

    return dir_name


def get_file_name(path):

    if "/" in path:
        file_name = path.rsplit("/", 1)[1]
    else:
        file_name = path

    return file_name


def append_file_path(path):

    append_path(get_dir_name(path))


def delete_path(path):

    if path_exists(path):
        if os.path.isfile(path):
            os.remove(path)
        elif os.path.isdir(path):
            shutil.rmtree(path)


def copy_path(source_path, dest_path):

    if path_exists(dest_path):
        delete_path(dest_path)
    shutil.copyfile(source_path, dest_path)


def save_table(path, df, sep="\t", header=True, index=False, fillna="None"):

    append_file_path(path)

    df = df.fillna(fillna)

    df = order_cols(df, get_col_order(df))

    df = order_rows(df)

    df.to_csv(path, sep=sep, header=header, index=index)


def load_table(path, sep="\t"):

    if path_exists(path):
        df = pd.read_csv(path, sep=sep, dtype=str)
        df = order_rows(df)
    else:
        df = None

    return df


def save_matrix(path, matrix, delim=","):

    append_file_path(path)

    np.savetxt(path, matrix, delimiter=delim)


def load_matrix(path, delim=","):

    if path_exists(path):
        matrix = np.loadtxt(path, delimiter=delim)
    else:
        matrix = None

    return matrix


def save_lst(path, val_lst):

    append_file_path(path)

    with open(path, "w") as file:
        for val in val_lst:
            file.write(f"{val}\n")


def load_lst(path):

    if path_exists(path):
        with open(path, "r") as file:
            line_lst = file.read().splitlines()
    else:
        line_lst = None

    return line_lst


def save_json(path, json_dict):

    append_file_path(path)

    with open(path, "w") as file:
        json.dump(json_dict, file)


def load_json(path):

    if path_exists(path):
        with open(path, "r") as file:
            json_dict = json.load(file)
    else:
        json_dict = None

    return json_dict


def unzip_file(in_path, out_path=None):

    if out_path is None:
        out_path = in_path.replace(".gz", "")

    with gzip.open(in_path, "rb") as file_in:
        with open(out_path, "wb") as file_out:
            shutil.copyfileobj(file_in, file_out)


def search_dir(dir_path, file_str):

    return [x for x in os.listdir(dir_path) if file_str in x]


def get_dir_path(dir_str=None, dir_path=None):

    if dir_path is None:
        dir_path = os.getcwd()

    if dir_str is not None:
        dir_path += f"/{dir_str}"

    return dir_path


def get_file_path(file_name, dir_str=None, dir_path=None, pre_str=True):

    file_path = get_dir_path(dir_str=dir_str, dir_path=dir_path)
    file_path += "/"
    if pre_str and dir_str != None:
        file_path += dir_str
        file_path += "_"
    file_path += file_name

    return file_path


def modify_coord_path(path, return_pdb=False, add_h=False):

    if return_pdb:
        path = path.replace(".cif", ".pdb")
        if add_h:
            path = path.replace(".pdb", ".h.pdb")

    return path


def get_core_path(
    pdb_code, chainid, modelid=None, dir_path=None, return_pdb=False, add_h=False
):
    pdb_id = f"{pdb_code}{chainid}"

    if modelid is not None:
        pdb_id += str(modelid)

    return modify_coord_path(
        get_file_path(
            f"{pdb_id}_{core_str}.cif",
            dir_str=core_str,
            dir_path=dir_path,
            pre_str=False,
        ),
        return_pdb=return_pdb,
        add_h=add_h,
    )


def get_rcsb_path(pdb_code, dir_path=None):

    return get_file_path(
        f"{pdb_code}.cif.gz", dir_str=rcsb_str, dir_path=dir_path, pre_str=False
    )


def get_sifts_path(pdb_code, dir_path=None):

    return get_file_path(
        f"{pdb_code}.xml.gz", dir_str=sifts_str, dir_path=dir_path, pre_str=False
    )


def get_renum_path(pdb_code, dir_path=None):

    return get_file_path(
        f"{pdb_code}_{renum_str}.cif",
        dir_str=renum_str,
        dir_path=dir_path,
        pre_str=False,
    )


def get_seq_path(uniprot_acc, dir_path=None):

    return get_file_path(
        f"{uniprot_acc}.fasta", dir_str=seq_str, dir_path=dir_path, pre_str=False
    )


def get_edia_path(pdb_code, dir_path=None):

    return get_file_path(
        f"{pdb_code}_{edia_str}.csv", dir_str=edia_str, dir_path=dir_path, pre_str=False
    )


def get_lig_path(lig, dir_path=None):

    return get_file_path(
        f"{lig}_{lig_str}.sdf", dir_str=lig_str, dir_path=dir_path, pre_str=False
    )


def get_eds_map_path(pdb_code, dir_path=None):

    return get_file_path(
        f"{pdb_code}_{map_str}.ccp", dir_str=map_str, dir_path=dir_path, pre_str=False
    )


def get_eds_diff_path(pdb_code, dir_path=None):

    return get_file_path(
        f"{pdb_code}_{diff_str}.ccp4",
        dir_str=diff_str,
        dir_path=dir_path,
        pre_str=False,
    )


def get_interf_path(pdb_code, chainid, interf, dir_path=None, return_pdb=False):

    return modify_coord_path(
        get_file_path(
            f"{pdb_code}_{chainid}_{interf}_{interf_str}.cif",
            dir_str=interf_str,
            dir_path=dir_path,
            pre_str=False,
        ),
        return_pdb=return_pdb,
    )


def get_pocket_path(pdb_id, pocket, dir_path=None):

    return get_file_path(
        f"{pdb_id}_{pocket}_{pocket_str}.pdb",
        dir_str=pocket_str,
        dir_path=dir_path,
        pre_str=False,
    )



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import re
import pandas as pd
import numpy as np
from datetime import datetime

from .col import (
    pdb_id_col,
    pdb_code_col,
    cf_col,
    prot_col,
    core_path_col,
    interf_path_col,
    pocket_path_col,
    modelid_col,
    chainid_col,
    interf_col,
    resid_col,
    data_col_lst,
    order_col_lst,
    reformat_col_lst,
    count_col_dict,
    index_col,
    total_col,
)
from .lst import (
    lst_inter,
    type_lst,
    str_to_lst,
    lst_to_str,
    format_val,
    lst_inter,
    sort_lst,
    move_end_lst,
    lst_nums,
    lst_unique,
)


def fix_val(val, return_str=False, return_int=False, return_float=False):

    if type(val) == str and is_int(val):
        if val[-2:] == ".0":
            val = str(val).replace(".0", "")

    return format_val(
        val, return_str=return_str, return_int=return_int, return_float=return_float
    )


def fix_query(query):

    return [fix_val(x) for x in type_lst(query)]


def fix_col(df, col):

    for index in list(df.index.values):

        df.at[index, col] = fix_val(df.at[index, col])

    return df


def mask_equal(df, col, query, reset_index=True):

    temp_df = df.copy(deep=True)

    query = fix_query(query)

    temp_df = fix_col(temp_df, col)

    temp_df = temp_df[temp_df[col].isin(query)]

    if reset_index:
        temp_df = temp_df.reset_index(drop=True)

    return temp_df


def mask_unequal(df, col, query, reset_index=True):

    temp_df = df.copy(deep=True)

    query = fix_query(query)

    temp_df = fix_col(temp_df, col)

    temp_df = temp_df[~temp_df[col].isin(query)]

    if reset_index:
        temp_df = temp_df.reset_index(drop=True)

    return temp_df


def mask_search(df, col, items, sep_txt, equal=True, reset_index=True):

    temp_df = df.copy(deep=True)

    item_lst = fix_query(items)

    temp_df = fix_col(temp_df, col)

    bool_lst = list()

    for row in df[col]:
        row_lst = str_to_lst(row, sep_txt=sep_txt, return_str=True)
        in_lst = not equal
        for item in item_lst:
            if str(item) in row_lst:
                in_lst = equal
        bool_lst.append(in_lst)

    temp_df = temp_df[bool_lst]

    if reset_index:
        temp_df = temp_df.reset_index(drop=True)

    return temp_df


def mask_greater(df, col, cutoff, reset_index=True):

    temp_df = df.copy(deep=True)

    temp_df[col] = temp_df[col].map(float)

    mask = temp_df[col] >= cutoff
    temp_df = temp_df.loc[mask, :]

    if reset_index:
        temp_df = temp_df.reset_index(drop=True)

    return temp_df


def mask_less(df, col, cutoff, reset_index=True):

    temp_df = df.copy(deep=True)

    temp_df[col] = temp_df[col].map(float)

    mask = temp_df[col] <= cutoff
    temp_df = temp_df.loc[mask, :]

    if reset_index:
        temp_df = temp_df.reset_index(drop=True)

    return temp_df


def mask_between(df, col, bottom, top, reset_index=True):

    return mask_greater(
        mask_less(df, col, top, reset_index=reset_index),
        col,
        bottom,
        reset_index=reset_index,
    )


def mask_matrix(matrix, row_lst, col_lst):

    return matrix[np.ix_(row_lst, col_lst)]


def lst_col(
    df, col, unique=False, return_str=False, return_int=False, return_float=False
):

    if unique:
        val_lst = list(df[col].unique())
        val_lst = sort_lst(val_lst)
        val_lst = move_end_lst(val_lst, ["Noise", "None"])
    else:
        val_lst = list(df[col].to_list())

    val_lst = format_val(
        val_lst,
        return_str=return_str,
        return_int=return_int,
        return_float=return_float,
    )

    return val_lst


def get_val_col(col, resid):

    return f"{col}_{resid}"


def get_col_val(col):

    return int(col.rsplit("_", 1)[1])


def get_col_col_lst(df, col):

    return [x for x in list(df.columns) if col in x]


def get_col_val_lst(df, col):

    return [get_col_val(x) for x in get_col_col_lst(df, col)]


def get_val_col_lst(df, val):

    return [x for x in list(df.columns) if val in x]


def reformat_val_table(df, cols):

    col_lst = type_lst(cols)

    id_lst = list()

    for col in list(df.columns):
        add = True
        for reformat_col in reformat_col_lst:
            if reformat_col in col:
                add = False
                break
        if add:
            id_lst.append(col)

    temp_df_lst = list()

    for col in col_lst:

        val_col_lst = get_val_col_lst(df, col)
        col_val_lst = get_col_val_lst(df, col)

        col_val_dict = make_dict(val_col_lst, col_val_lst)

        temp_df = pd.melt(
            df,
            id_vars=id_lst,
            value_vars=val_col_lst,
            var_name=resid_col,
            value_name=col,
        ).drop_duplicates()

        temp_df[resid_col] = temp_df[resid_col].map(col_val_dict)

        temp_df_lst.append(temp_df)

    if len(temp_df_lst) == 1:
        df = temp_df_lst[0]
    else:
        df = merge_tables(temp_df_lst[0], temp_df_lst[1])

        if len(temp_df_lst) > 2:
            i_lst = lst_nums(2, len(temp_df_lst) - 1)

            for i in i_lst:
                df = merge_tables(df, temp_df_lst[i])

    return df


def get_col_order(df):

    df_col_lst = list(df.columns)

    col_lst = list()
    for order_col in order_col_lst:
        if order_col in df_col_lst:
            col_lst.append(order_col)
        if order_col in data_col_lst:
            for col in get_val_col_lst(df, order_col):
                if col not in col_lst:
                    col_lst.append(col)

    for col in df_col_lst:
        if col not in col_lst:
            col_lst.append(col)

    return col_lst


def order_cols(df, col_lst):

    df = df.reindex(columns=col_lst)

    return df


def order_rows(df, col_lst=None, reset_index=False):

    df_col_lst = list(df.columns)

    if col_lst is None:
        col_lst = list()

        if interf_path_col in df_col_lst:
            col_lst.append(interf_path_col)
        elif pocket_path_col in df_col_lst:
            col_lst.append(pocket_path_col)
        elif core_path_col in df_col_lst:
            col_lst.append(core_path_col)

        if pdb_id_col in df_col_lst:
            col_lst.append(pdb_id_col)
        if pdb_code_col in df_col_lst:
            col_lst.append(pdb_code_col)
        if modelid_col in df_col_lst:
            col_lst.append(modelid_col)
        if chainid_col in df_col_lst:
            col_lst.append(chainid_col)
        if interf_col in df_col_lst:
            col_lst.append(interf_col)

    if len(col_lst) > 0:
        df = df.sort_values(by=col_lst)

    if reset_index:
        df = df.reset_index(drop=True)

    return df


def make_dict(lst_1, lst_2):

    lst_dict = dict(zip(lst_1, lst_2))

    return lst_dict


def merge_dicts(dict_lst):

    fin_dict = {}

    for curr_dict in dict_lst:

        fin_dict = {**fin_dict, **curr_dict}

    return fin_dict


def rename_dict_key(the_dict, old_key, new_key):

    the_dict[new_key] = the_dict.pop(old_key)

    return the_dict


def get_str_num(str_num):

    return int("".join(filter(str.isdigit, str_num)))


def replace_str(val, term, replace=""):

    term_lst = type_lst(term)

    for term in term_lst:
        if term in val:
            val = val.replace(term, replace)

    return val


def build_count_dict(val_lst):

    count_dict = {}
    for val in val_lst:
        if val in count_dict:
            count_dict[val] += 1
        else:
            count_dict[val] = 1

    return count_dict


def build_col_count_dict(df, col, col_lst=None, return_str=False, return_int=False):

    if col_lst is not None:
        col_lst = type_lst(col_lst)
        col_lst.append(col)
        df = df.loc[:, col_lst]
        df = df.drop_duplicates()

    count_lst = lst_col(df, col, return_str=return_str, return_int=return_int)
    count_dict = build_count_dict(count_lst)

    return count_dict


def lst_by_freq(val_lst):

    count_dict = build_count_dict(val_lst)

    sorted_vals = sorted(count_dict, key=count_dict.get, reverse=True)

    return sorted_vals


def get_col_most_common(df, col, n=None):

    col_lst = lst_unique(lst_by_freq(lst_col(df, col)))

    if n is not None:
        col_lst[:n]
        if n == 1:
            col_lst = col_lst[0]

    return col_lst


def extract_str(val):

    if val is not None:
        val = lst_to_str([x for x in str(val) if x.isalpha()], join_txt="", empty="")

    return val


def extract_int(val):

    if val is not None:
        val = int(re.search(r"\d+", str(val)).group())

    return val


def is_str(val):

    if type(val) == str:
        status = len(extract_str(val)) == len(val)
    else:
        status = False

    return status


def is_int(val):

    return not is_str(val)


def explode_table(df, cols, sep=","):

    temp_df = df.copy(deep=True)

    temp_df[cols] = temp_df[cols].str.split(sep)

    temp_df = temp_df.explode(cols)
    temp_df = temp_df.reset_index(drop=True)

    return temp_df


def merge_tables(left_df, right_df, how="left"):

    col_lst = lst_inter(list(left_df.columns), list(right_df.columns))

    for col in col_lst:
        left_df[col] = left_df[col].map(str)
        right_df[col] = right_df[col].map(str)

    return left_df.merge(
        right_df,
        on=col_lst,
        how=how,
    )


def build_label_dict(
    df,
    col,
    return_str=False,
    return_int=False,
    count_chain=True,
    count_pdb=False,
    count_cf=False,
):

    df_col_lst = list(df.columns)

    if interf_path_col in df_col_lst:
        interf_dict = build_col_count_dict(
            df,
            col,
            col_lst=[interf_path_col],
            return_str=return_str,
            return_int=return_int,
        )
    if pocket_path_col in df_col_lst:
        pocket_dict = build_col_count_dict(
            df,
            col,
            col_lst=[pocket_path_col],
            return_str=return_str,
            return_int=return_int,
        )

    if count_chain:
        if pdb_id_col in df_col_lst:
            chain_col_lst = [pdb_id_col]
        else:
            chain_col_lst = [core_path_col, modelid_col, chainid_col]

        chain_dict = build_col_count_dict(
            df, col, col_lst=chain_col_lst, return_str=return_str, return_int=return_int
        )

    if count_pdb:
        if pdb_code_col in df_col_lst:
            entry_dict = build_col_count_dict(
                df,
                col,
                col_lst=[pdb_code_col],
                return_str=return_str,
                return_int=return_int,
            )

    if count_cf:
        if cf_col in df_col_lst:
            cf_dict = build_col_count_dict(
                df, col, col_lst=[cf_col], return_str=return_str, return_int=return_int
            )

    val_lst = lst_col(
        df, col, unique=True, return_str=return_str, return_int=return_int
    )

    label_dict = dict()

    for val in val_lst:
        name = val
        name += " ("

        if interf_path_col in df_col_lst:
            name += f"I={interf_dict[str(val)]}; "
        if pocket_path_col in df_col_lst:
            name += f"S={pocket_dict[str(val)]}; "

        if count_chain:
            name += f"N={chain_dict[str(val)]}"
        if count_pdb:
            if pdb_code_col in df_col_lst:
                if count_chain:
                    name += "; "
                name += f"PDB={entry_dict[str(val)]}"
        if count_cf:
            if cf_col in df_col_lst:
                if count_chain or (count_pdb and pdb_code_col in df_col_lst):
                    name += "; "
                name += f"CF={cf_dict[str(val)]}"

        name += ")"
        label_dict[val] = name

    return label_dict


def build_label_color_dict(
    df,
    col,
    color_dict,
    label_order=None,
    return_str=False,
    return_int=False,
    count_chain=True,
    count_pdb=False,
    count_cf=False,
):

    label_dict = build_label_dict(
        df,
        col,
        return_str=return_str,
        return_int=return_int,
        count_chain=count_chain,
        count_pdb=count_pdb,
        count_cf=count_cf,
    )

    if label_order is None:
        label_order = list(label_dict.keys())

    label_color_dict = dict()
    for label in label_order:
        label_color_dict[label_dict[label]] = color_dict[label]

    return label_color_dict


def get_ncols(label_lst):

    total = len(label_lst)

    if 1 < total <= 5:
        cols = total
    else:
        cols = 5

    return cols


def title_str(str_val):

    if "_" in str_val:
        str_val = str_val.replace("_", " ")

    return str_val.title()


def get_val_index_lst(df, col, val):

    temp_df = mask_equal(df, col, val, reset_index=False)

    return list(temp_df.index.values)


def build_count_table(df, cols):

    col_lst = type_lst(cols)

    temp_df = df.copy(deep=True)

    temp_df = temp_df.rename(columns=count_col_dict)

    df_col_lst = list(temp_df.columns)

    count_df_lst = list()

    for col in list(count_col_dict.values()):
        if col in df_col_lst:
            count_df_lst.append(
                pd.pivot_table(temp_df, index=col_lst, values=col, aggfunc="nunique")
                .fillna("-")
                .reset_index()
            )
    total_counts = len(count_df_lst)
    if total_counts == 0:
        temp_df = temp_df.reset_index()
        temp_df = temp_df.rename(columns={index_col: total_col})
        count_df = (
            pd.pivot_table(temp_df, index=col_lst, values=total_col, aggfunc="nunique")
            .fillna("-")
            .reset_index()
        )
    else:
        count_df = count_df_lst[0]
        if total_counts > 0:
            for i in lst_nums(1, total_counts - 1):
                count_df = merge_tables(count_df, count_df_lst[i])

        if prot_col in df_col_lst:
            prot_df = pd.crosstab(
                columns=temp_df[prot_col], index=[temp_df[col] for col in col_lst]
            )
            for col in list(prot_df.columns):
                prot_df[col] = prot_df[col].replace({0: "-"})

            prot_df = prot_df.reset_index()

            count_df = merge_tables(count_df, prot_df)

    for col in col_lst:
        count_df[col] = count_df[col].map(str)

    return count_df


def get_df_at_index(df, index):

    return df.loc[type_lst(index), :]


def convert_col_percent(df, col):

    df[col] = (df[col] / df[col].sum()) * 100

    return df


# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import numpy as np
from tqdm import tqdm
import itertools
import math

from ..functions import *


def calc_flip_dist(norm_dist, resid_dict, index_dict, norm_dict, flip_dict, mean=False):

    if norm_dist <= 2:
        flip_dist = norm_dist
    else:
        flip_dist_lst = list()
        bb_resid_lst = resid_dict[phi_col]
        min_resid = min(bb_resid_lst)
        max_resid = max(bb_resid_lst)
        for flip_diff in list(flip_dict.keys()):
            for index, resid in enumerate(bb_resid_lst):
                if resid != min_resid and resid != max_resid:
                    curr_resid = resid
                    next_resid = bb_resid_lst[index + 1]
                    if abs(extract_int(next_resid) - extract_int(curr_resid)) < 2:
                        if curr_resid in list(
                            index_dict[psi_col].keys()
                        ) and next_resid in list(index_dict[phi_col].keys()):
                            curr_index = index_dict[psi_col][curr_resid]
                            next_index = index_dict[phi_col][next_resid]

                            temp_psi_vals = np.copy(norm_dict[psi_col])
                            temp_phi_vals = np.copy(norm_dict[phi_col])

                            temp_psi_vals[curr_index] = flip_dict[flip_diff][psi_col][
                                curr_index
                            ]
                            temp_phi_vals[next_index] = flip_dict[flip_diff][phi_col][
                                next_index
                            ]

                            temp_flip_vals = np.array([])

                            temp_flip_vals = np.concatenate(
                                (temp_flip_vals, temp_psi_vals), axis=0
                            )
                            temp_flip_vals = np.concatenate(
                                (temp_flip_vals, temp_phi_vals), axis=0
                            )

                            for dih_col in dih_col_lst:
                                if dih_col != psi_col and dih_col != phi_col:
                                    norm_angle_vals = norm_dict[dih_col]
                                    if len(norm_angle_vals) > 0:
                                        temp_flip_vals = np.concatenate(
                                            (temp_flip_vals, norm_angle_vals), axis=0
                                        )
                            if mean:
                                temp_flip_dist = np.mean(temp_flip_vals)
                            else:
                                temp_flip_dist = np.max(temp_flip_vals)
                            flip_dist_lst.append(temp_flip_dist)

        if len(flip_dist_lst) == 0:
            flip_dist = norm_dist
        else:
            flip_dist = min(min(flip_dist_lst), norm_dist)

    return flip_dist


def calc_dih_dist(i, j, i_df, j_df, flip_diff=180):

    resid_dict = dict()

    for dih_col in dih_col_lst:

        resid_dict[dih_col] = get_col_val_lst(i_df, dih_col)

    norm_dict = dict()
    flip_dict = dict()

    for dih_col in dih_col_lst:

        norm_dict[dih_col] = np.array([])

    flip_diff_lst = type_lst(flip_diff)

    for flip_diff in flip_diff_lst:
        flip_dict[flip_diff] = {phi_col: np.array([]), psi_col: np.array([])}

    index_dict = dict()

    for dih_col in dih_col_lst:

        resid_lst = resid_dict[dih_col]

        index_dict[dih_col] = dict()
        index = 0

        for resid in resid_lst:

            val_col = get_val_col(dih_col, resid)

            norm_val_i = float(i_df.at[i, val_col])
            norm_val_j = float(j_df.at[j, val_col])

            if norm_val_i != 999.00 and norm_val_j != 999.00:

                norm_dist = 2 * (
                    1 - math.cos((math.radians(norm_val_i) - math.radians(norm_val_j)))
                )

                norm_dict[dih_col] = np.append(norm_dict[dih_col], norm_dist)

                for flip_diff in flip_diff_lst:

                    if dih_col == psi_col or dih_col == phi_col:
                        flip_dist = 2 * (
                            1
                            - math.cos(
                                (
                                    math.radians(norm_val_i + flip_diff)
                                    - math.radians(norm_val_j)
                                )
                            )
                        )

                        flip_dict[flip_diff][dih_col] = np.append(
                            flip_dict[flip_diff][dih_col], flip_dist
                        )

                index_dict[dih_col][resid] = index
                index += 1

    norm_vals = np.array([])

    for dih_col in dih_col_lst:
        norm_vals = np.concatenate((norm_vals, norm_dict[dih_col]), axis=0)

    if len(norm_vals) == 0:
        max_norm_dist = 4.0
        mean_norm_dist = 4.0
        max_flip_dist = 4.0
        mean_flip_dist = 4.0
    else:
        max_norm_dist = np.max(norm_vals)
        mean_norm_dist = np.mean(norm_vals)
        max_flip_dist = calc_flip_dist(
            max_norm_dist, resid_dict, index_dict, norm_dict, flip_dict, mean=False
        )
        mean_flip_dist = calc_flip_dist(
            mean_norm_dist, resid_dict, index_dict, norm_dict, flip_dict, mean=True
        )

    max_norm_result = (i, j, max_norm_dist)
    mean_norm_result = (i, j, mean_norm_dist)
    max_flip_result = (i, j, max_flip_dist)
    mean_flip_result = (i, j, mean_flip_dist)

    return (max_norm_result, mean_norm_result, max_flip_result, mean_flip_result)


def build_dih_matrix(
    fit_df,
    max_norm_path,
    mean_norm_path=None,
    max_flip_path=None,
    mean_flip_path=None,
    pred_df=None,
    flip_diff=180,
):

    fit_df = order_rows(fit_df)
    j_df = fit_df.copy(deep=True)

    if pred_df is None:
        i_df = fit_df.copy(deep=True)
    else:
        pred_df = order_rows(pred_df)
        i_df = pred_df.copy(deep=True)

    i_index_lst = list(i_df.index.values)
    j_index_lst = list(j_df.index.values)

    if pred_df is not None:
        index_pairs = itertools.product(i_index_lst, j_index_lst)
    else:
        index_pairs = itertools.combinations(i_index_lst, 2)

    max_norm_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))
    mean_norm_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))
    max_flip_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))
    mean_flip_matrix = np.zeros((len(i_index_lst), len(j_index_lst)))

    matrix_lst = [max_norm_matrix, mean_norm_matrix, max_flip_matrix, mean_flip_matrix]

    for i_index, j_index in tqdm(
        index_pairs, desc="Building dihedral matrix", position=0, leave=True
    ):
        dih_result = calc_dih_dist(
            i_index,
            j_index,
            i_df,
            j_df,
            flip_diff=flip_diff,
        )

        result_lst = [
            dih_result[0],
            dih_result[1],
            dih_result[2],
            dih_result[3],
        ]

        for result, matrix in zip(result_lst, matrix_lst):

            i_index = result[0]
            j_index = result[1]
            dist = result[2]

            matrix[i_index, j_index] = dist

            if pred_df is None:
                matrix[j_index, i_index] = dist

    save_matrix(max_norm_path, max_norm_matrix)

    if mean_norm_path is not None:
        save_matrix(mean_norm_path, mean_norm_matrix)

    if max_flip_path is not None:
        save_matrix(max_flip_path, max_flip_matrix)

    if mean_flip_path is not None:
        save_matrix(mean_flip_path, mean_flip_matrix)

    print("Built dihedral matrix!")



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
from pandas.core.frame import DataFrame
import seaborn as sns
import matplotlib.pyplot as plt
from matplotlib_venn import venn2

from .color import get_lst_colors, gray_hex, black_hex, change_hex_alpha

from .lst import format_val

from .path import append_file_path

from .table import mask_equal, lst_col, make_dict, build_label_dict, get_ncols

from .col import pdb_id_col

grid_hex = change_hex_alpha(gray_hex, 0.25)


def prep_plot_col(
    df,
    col,
    color_palette=None,
    rename_vals=None,
    order_lst=None,
    label_count=False,
    count_chain=True,
    count_pdb=False,
    count_cf=False,
    return_palette=True,
):

    df[col] = df[col].map(str)

    for index in list(df.index.values):

        df.at[index, col] = str(df.at[index, col]).split(" (N=")[0]

    if rename_vals is not None:
        if type(rename_vals) == dict:
            rename_dict = rename_vals
        elif type(rename_vals) == list:
            val_lst = lst_col(df, col, unique=True, return_str=True)
            rename_lst = format_val(rename_vals, return_str=True)
            rename_dict = make_dict(val_lst, rename_lst)
        df[col] = df[col].map(rename_dict)

    if order_lst is not None:
        order_lst = format_val(order_lst, return_str=True)
        df = mask_equal(df, col, order_lst)
        return_lst = order_lst.copy()
    else:
        return_lst = lst_col(df, col, unique=True, return_str=True)

    if label_count:
        col_dict = build_label_dict(
            df,
            col,
            return_str=True,
            count_chain=count_chain,
            count_pdb=count_pdb,
            count_cf=count_cf,
        )

        df[col] = df[col].map(col_dict)
        for i, row in enumerate(return_lst):
            return_lst[i] = col_dict[row]

    if type(color_palette) == dict:
        return_dict = dict()
        for val in return_lst:
            return_dict[val] = color_palette[val.split(" (")[0]]
    else:
        return_dict = color_palette

    if return_palette:
        return df, return_lst, return_dict
    else:
        return (
            df,
            return_lst,
        )


def make_legend_plot(
    plot_path,
    legend_dict,
    plot_width=2,
    plot_height=2,
    font_size=7,
    marker_shape="s",
    marker_size=3,
    legend_cols=None,
    legend_title=None,
    color_text=False,
):

    fig, ax = plt.subplots()

    fig.set_size_inches(plot_width, plot_height)

    handles = [
        plt.plot(
            [],
            [],
            marker=marker_shape,
            ls="",
            markersize=marker_size,
            markeredgewidth=0,
            markerfacecolor=color,
            label=label,
            color=color,
        )[0]
        for label, color in legend_dict.items()
    ]

    if legend_cols is None:
        legend_cols = get_ncols(list(legend_dict.keys()))

    label_color = None
    if color_text:
        label_color = "linecolor"

    legend = ax.legend(
        handles=handles,
        fontsize=font_size,
        ncol=legend_cols,
        frameon=False,
        loc="center",
        bbox_to_anchor=(0.5, 0.5),
        title=legend_title,
        labelcolor=label_color,
    )

    if legend_title is not None:
        plt.setp(legend.get_title(), fontsize=font_size)

    bbox_extra_artists = (legend,)

    plt.axis("off")

    append_file_path(plot_path)

    plt.savefig(
        plot_path,
        format="pdf",
        bbox_extra_artists=bbox_extra_artists,
        bbox_inches="tight",
        pad_inches=0.0,
        dpi=600,
    )
    plt.close()


def make_venn_plot(
    lst_1,
    lst_2,
    plot_path,
    label_1=None,
    label_2=None,
    color_1=None,
    color_2=None,
    color_inter=None,
    count_color="black",
    plot_title=None,
    plot_height=2,
    plot_width=2,
    font_size=7,
    alpha=0.75,
):

    if color_1 is None:
        color_1 = gray_hex
    if color_2 is None:
        color_2 = gray_hex
    if color_inter is None:
        color_inter = black_hex

    fig, ax = plt.subplots()

    fig.set_size_inches(plot_width, plot_height)

    total = len(set(lst_1 + lst_2))

    v = venn2(
        [set(lst_1), set(lst_2)],
        set_labels=(label_1, label_2),
        set_colors=(color_1, color_2),
        alpha=alpha,
        subset_label_formatter=lambda x: f"{x}\n({(x/total):1.0%})",
        ax=ax,
    )

    bbox_extra_artists = tuple()

    for text in v.set_labels:
        text.set_fontsize(font_size)
        bbox_extra_artists += (text,)

    for text in v.subset_labels:

        if text.get_text() == "0\n(0%)":
            text.set_text("")
        else:
            text.set_fontsize(font_size * 0.75)
            text.set_color(count_color)
            bbox_extra_artists += (text,)

    if plot_title is not None:
        title = fig.suptitle(plot_title, fontsize=font_size)
        bbox_extra_artists += (title,)

    append_file_path(plot_path)

    plt.savefig(
        plot_path,
        format="pdf",
        bbox_extra_artists=bbox_extra_artists,
        bbox_inches="tight",
        pad_inches=0.0,
        dpi=600,
    )
    plt.close()


def make_stacked_barplot(
    plot_df,
    col_col,
    hue_col,
    plot_path,
    col_order=None,
    rename_col=None,
    col_count=False,
    hue_order=None,
    rename_hue=None,
    hue_count=False,
    hue_palette=None,
    x_str=None,
    y_str=None,
    font_size=7,
    plot_height=2,
    plot_width=2,
    line_width=0.5,
    show_legend=True,
    legend_pad=10,
    legend_cols=None,
    bar_width=0.5,
    bar_alpha=1,
    count_chain=True,
    count_pdb=False,
    count_cf=False,
    id_col=None,
    show_barh=False,
):

    df = plot_df.copy(deep=True)

    if id_col is None:
        id_column = pdb_id_col

    df, col_lst = prep_plot_col(
        df,
        col_col,
        rename_vals=rename_col,
        order_lst=col_order,
        label_count=col_count,
        count_chain=count_chain,
        count_pdb=count_pdb,
        count_cf=count_cf,
        return_palette=False,
    )

    df, hue_lst, hue_palette = prep_plot_col(
        df,
        hue_col,
        color_palette=hue_palette,
        rename_vals=rename_hue,
        order_lst=hue_order,
        label_count=hue_count,
        count_chain=count_chain,
        count_pdb=count_pdb,
        count_cf=count_cf,
    )

    hue_color_dict = get_lst_colors(hue_lst, palette=hue_palette, return_dict=True)

    sns.set_context("paper")
    sns.set_style("whitegrid")
    sns.set_palette(list(hue_color_dict.values()))

    df = pd.pivot_table(
        df,
        index=col_col,
        columns=hue_col,
        values=id_column,
        aggfunc="nunique",
    ).fillna(0)

    df = df.reindex(index=col_lst)
    df = df.reindex(columns=hue_lst)

    if show_barh:
        plot_kind = "barh"
        grid_axis = "y"
        if x_str is None:
            x_str = "% Structures"
        if y_str is None:
            y_str = col_col
    else:
        plot_kind = "bar"
        grid_axis = "x"
        if x_str is None:
            x_str = col_col
        if y_str is None:
            y_str = "% Structures"

    df.apply(lambda x: x / sum(x) * 100, axis=1).plot(
        kind=plot_kind,
        stacked=True,
        linewidth=0,
        width=bar_width,
        alpha=bar_alpha,
        figsize=(plot_width, plot_height),
        legend=show_legend,
    )

    if show_barh:
        plt.xticks(fontsize=font_size * 0.75)
    else:
        plt.xticks(fontsize=font_size * 0.75, rotation=45, ha="right")

    plt.yticks(fontsize=font_size * 0.75)
    plt.grid(axis=grid_axis, color=grid_hex, linewidth=line_width)

    x_label = plt.xlabel(x_str, fontsize=font_size)
    y_label = plt.ylabel(y_str, fontsize=font_size)

    bbox_extra_artists = (x_label, y_label)

    if show_legend or type(show_legend) == dict:

        if legend_cols is None:
            legend_cols = get_ncols(hue_lst)

        if type(show_legend) == dict:
            handles = [
                plt.plot(
                    [],
                    [],
                    marker="o",
                    ls="",
                    markeredgewidth=0,
                    markersize=3,
                    markerfacecolor=show_legend[hue],
                    label=hue,
                    color=hue,
                )[0]
                for hue in list(show_legend.keys())
            ]
        else:
            handles = [
                plt.plot(
                    [],
                    [],
                    marker="o",
                    ls="",
                    markeredgewidth=0,
                    markersize=3,
                    markerfacecolor=hue_color_dict[hue],
                    label=hue,
                )[0]
                for hue in hue_lst
            ]

        legend = plt.legend(
            handles=handles,
            fontsize=font_size * 0.75,
            ncol=legend_cols,
            loc="upper center",
            frameon=False,
            bbox_to_anchor=(0.5, 0),
            borderaxespad=legend_pad,
        )

        bbox_extra_artists += (legend,)

    sns.despine(left=True)

    plt.savefig(
        plot_path,
        format="pdf",
        bbox_inches="tight",
        bbox_extra_artists=bbox_extra_artists,
        dpi=600,
    )
    plt.close()



import os
import sys
from os import listdir
from os.path import isfile, join
from pathlib import Path
import Bio
from Bio.PDB import *
import pandas as pd  # a great dataframe
import numpy as np  # a large collection of high-level mathematical functions to operate on these arrays
import math
import re  # a regular expression operations
import argparse  # parser for arguments
import time  # time module
from datetime import date
import gzip
import shutil  # compressing module
import xml.etree.ElementTree as ET  # xml parser
import urllib  # urllib
from string import punctuation
import requests
import tqdm  # a progress bar
import socket

from functools import partial
from lxml import html
import requests
from concurrent.futures import as_completed, ProcessPoolExecutor, ThreadPoolExecutor

# exception_AccessionIDs = ["P42212", "Q17104", "Q27903", "Q93125", "P03069", "D3DLN9", "Q96UT3", "P0ABE7", "P00192",
#                           "P76805", "Q8XCE3", "P00720", "Q38170", "Q94N07", "P0AEX9", "P02928", "Q2M6S0"]
socket.setdefaulttimeout(660)
current_directory = os.getcwd()


# # maybe unnecessary
# from multiprocessing import Process
# from multiprocessing import Pool
# from multiprocessing.pool import ThreadPool

# current_directory = os.getcwd()
# default_input_path_to_mmCIF = current_directory + "/mmCIF"
# default_input_path_to_PDB = current_directory + "/PDB"
# default_input_path_to_SIFTS = current_directory + "/SIFTS"
# default_output_path_to_mmCIF = current_directory + "/output_mmCIF"
# default_output_path_to_PDB = current_directory + "/output_PDB"
#
# default_input_path_to_mmCIF_assemblies = current_directory + "/mmCIF_assembly"
# default_input_path_to_PDB_assemblies = current_directory + "/PDB_assembly"
# default_output_path_to_mmCIF_assemblies = current_directory + "/output_mmCIF_assembly"
# default_output_path_to_PDB_assemblies = current_directory + "/output_PDB_assembly"

# current_directory = os.getcwd()
# default_mmCIF_num = 50000
# default_PDB_num = 5000
# gzip_mode = "on"



from src.download.modules import *


def handling_chain_numbering_clashes(df_PDBe_PDB_UniProt, exception_AccessionIDs):
    chains_to_change = set()
    chains_to_change_one_to_end = set()
    AccessionIDs = set()
    chain_AccessionID_dict = dict()

    for PDBe_num_UniProt_PDB_accession in df_PDBe_PDB_UniProt["Three_Rows_CIF_Num_Uni"]:
        if type(PDBe_num_UniProt_PDB_accession[4]) == float:
            continue
        chains_to_change.add(PDBe_num_UniProt_PDB_accession[3][2])
        chains_to_change_one_to_end.add(PDBe_num_UniProt_PDB_accession[2][2])
        AccessionIDs.add(PDBe_num_UniProt_PDB_accession[4])

    for chains in chains_to_change:
        accessions_in_chain = set()
        for PDBe_num_UniProt_PDB_accession in df_PDBe_PDB_UniProt["Three_Rows_CIF_Num_Uni"]:
            if chains == PDBe_num_UniProt_PDB_accession[3][2]:
                if PDBe_num_UniProt_PDB_accession[4] is not np.nan:
                    accessions_in_chain.add(PDBe_num_UniProt_PDB_accession[4])
        chain_AccessionID_dict[chains] = accessions_in_chain

    tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = list()
    combined_tuple_PDBe_UniProt_AccessionID = list()
    longest_AccessionID_list = list()
    clash = 0

    for chain_accession in chain_AccessionID_dict.items():
        chains_to_change_for_AccessionID = list()
        longest_AccessionID = None
        longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = list()
        if len(chain_accession[1]) > 1:
            for accessions in chain_accession[1]:
                tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = list()
                target_UniProt_numbers_in_chain = list()
                diff_another_UniProt_numbers_in_same_chain = list()
                diff_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = list()

                for PDBe_num_UniProt_PDB_accession in df_PDBe_PDB_UniProt["Three_Rows_CIF_Num_Uni"]:
                    if PDBe_num_UniProt_PDB_accession[4] == accessions and PDBe_num_UniProt_PDB_accession[3][2] == chain_accession[0] and \
                            PDBe_num_UniProt_PDB_accession[4] is not np.nan:
                        tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID.append(
                            (PDBe_num_UniProt_PDB_accession[0], PDBe_num_UniProt_PDB_accession[2], PDBe_num_UniProt_PDB_accession[4]))
                        target_UniProt_numbers_in_chain.append(PDBe_num_UniProt_PDB_accession[2])
                        chains_to_change_for_AccessionID.append(PDBe_num_UniProt_PDB_accession[3][2])
                    if PDBe_num_UniProt_PDB_accession[4] != accessions and PDBe_num_UniProt_PDB_accession[3][2] == chain_accession[0] and \
                            PDBe_num_UniProt_PDB_accession[4] is not np.nan:
                        diff_another_UniProt_numbers_in_same_chain.append(PDBe_num_UniProt_PDB_accession[2])
                        diff_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID.append(
                            (PDBe_num_UniProt_PDB_accession[0], PDBe_num_UniProt_PDB_accession[2], PDBe_num_UniProt_PDB_accession[4]))

                for target_Uni in target_UniProt_numbers_in_chain:
                    for diff_Uni in diff_another_UniProt_numbers_in_same_chain:
                        if target_Uni[0] == diff_Uni[0]:
                            clash = 1

                if accessions not in exception_AccessionIDs:
                    if len(longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID) < len(
                            tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID):
                        longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID
                        longest_AccessionID = accessions

                if longest_AccessionID is None:
                    if len(longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID) < len(
                            tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID):
                        longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID
                        longest_AccessionID = accessions

            if clash == 1:
                combined_tuple_PDBe_UniProt_AccessionID.extend(longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID)
                longest_AccessionID_list.append(longest_AccessionID)
            else:
                combined_tuple_PDBe_UniProt_AccessionID.extend(longest_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID)
                combined_tuple_PDBe_UniProt_AccessionID.extend(diff_tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID)
        else:
            for accessions in chain_accession[1]:
                tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID = list()
                target_UniProt_numbers_in_chain = list()

                for PDBe_num_UniProt_PDB_accession in df_PDBe_PDB_UniProt["Three_Rows_CIF_Num_Uni"]:
                    if PDBe_num_UniProt_PDB_accession[4] == accessions and PDBe_num_UniProt_PDB_accession[3][2] == chain_accession[0] and \
                            PDBe_num_UniProt_PDB_accession[4] is not np.nan:
                        tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID.append((PDBe_num_UniProt_PDB_accession[0],
                                                                                         PDBe_num_UniProt_PDB_accession[2],
                                                                                         PDBe_num_UniProt_PDB_accession[4]))
                        target_UniProt_numbers_in_chain.append(PDBe_num_UniProt_PDB_accession[2])
                        chains_to_change_for_AccessionID.append(PDBe_num_UniProt_PDB_accession[3][2])
            combined_tuple_PDBe_UniProt_AccessionID.extend(tuple_PDBe_for_UniProt_and_tuple_UniProt_for_AccessionID)

    return [chains_to_change, combined_tuple_PDBe_UniProt_AccessionID, AccessionIDs, longest_AccessionID_list, chains_to_change_one_to_end]



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

sw1_col = "SW1"
sw2_col = "SW2"
id_col = "id"

core_path_col = "core_path"
rcsb_path_col = "rcsb_path"
renum_path_col = "renum_path"
rcsb_assembly_path_col = "rcsb_assembly_path"
renum_assembly_path_col = "renum_assembly_path"
sifts_path_col = "sifts_path"
edia_path_col = "edia_path"
interf_path_col = "interf_path"
pocket_path_col = "pocket_path"

modelid_col = "modelid"
chainid_col = "chainid"
resid_col = "resid"
resname_col = "resname"
atomid_col = "atomid"

pdb_id_col = "pdb_id"
pdb_code_col = "pdb_code"
swiss_id_col = "swiss_id"
uniprot_id_col = "uniprot_id"
pfam_col = "pfam"
prot_col = "prot_name"
method_col = "method"
resolution_col = "resolution"
r_factor_col = "r_factor"
seq_col = "sequence"
len_col = "length"
range_col = "range"

len_a_col = "length_a"
len_b_col = "length_b"
len_c_col = "length_c"
ang_a_col = "angle_alpha"
ang_b_col = "angle_beat"
ang_g_col = "angle_gamma"
space_col = "space_group"
cf_col = "crystal_form"

mut_status_col = "mutation_status"
mut_pos_col = "mutation_position"

interf_id_col = "interface_id"
interf_col = "interface"
interf_area_col = "interface_area"
cb_cont_col = "cb_contacts"
atomid_cont_col = "atomid_contacts"
total_cb_cont_col = "total_cb_contacts"
total_atomid_cont_col = "total_atomid_contacts"
interf_cont_col = "interface_contacts"
cb_dist_col = "cb_distance"
iso_col = "isologous"

pocket_id_col = "pocket_id"
pocket_col = "pocket"
pocket_site_col = "pocket_site"
pocket_volume_col = "pocket_volume"
pocket_score_col = "pocket_score"
pocket_status_col = "pocket_status"
pocket_type_col = "pocket_type"
pocket_cont_col = "pocket_contacts"

bio_lig_col = "biological_ligand"
ion_lig_col = "ion_ligand"
pharm_lig_col = "pharmacological_ligand"
chem_lig_col = "chemical_ligand"
mod_lig_col = "modification_ligand"
mem_lig_col = "membrane_ligand"
pocket_lig_col = "pocket_ligand"

gene_class_col = "gene_class"
nuc_class_col = "nucleotide_class"
mut_class_col = "mut_class"
pharm_class_col = "pharm_class"
match_class_col = "match_class"
prot_class_col = "protein_class"
interf_class_col = "interf_class"
pocket_class_col = "pocket_class"

bound_lig_col = "bound_ligand"
bound_prot_col = "bound_protein"
bound_prot_swiss_id_col = "bound_protein_swiss_id"

pharm_lig_site_col = "pharmacological_ligand_site"
pharm_lig_match_col = "pharmarmacological_ligand_match"

bound_prot_pfam_col = "bound_protein_pfam"
bound_prot_site_col = "bound_protein_site"

bio_lig_cont_col = "biological_ligand_contacts"
pharm_lig_cont_col = "pharmacological_ligand_contacts"
bound_prot_cont_col = "bound_protein_contacts"

bound_prot_chainid_col = "bound_protein_chainid"
bound_interf_chainid_col = "bound_interf_chainid"

edia_col = "edia_score"
b_factor_col = "b_factor"

phi_col = "phi"
psi_col = "psi"
omega_col = "omega"
chi1_col = "chi1"
altchi1_col = "altchi1"
chi2_col = "chi2"
altchi2_col = "altchi2"
chi3_col = "chi3"
chi4_col = "chi4"
chi5_col = "chi5"

complete_col = "complete"
rama_col = "rama"
rotamer_col = "rotamer"
bb_resid_col = "bb_resid"
sc_resid_col = "sc_resid"
bb_seq_col = "bb_seq"
sc_seq_col = "sc_seq"
bb_len_col = "bb_length"
sc_len_col = "sc_length"

cluster_col = "cluster"
nn_dist_col = "nn_dist"
constr_dist_col = "constraint_distance"

nn_cutoff_col = "nn_cutoff"
constr_cutoff_col = "constraint_cutoff"

total_col = "total"
total_chain_col = "total_chain"
total_entry_col = "total_entry"
total_cf_col = "total_cf"

loop_col = "loop"
silh_col = "silhouette_score"
simi_col = "similarity_score"
total_complete_col = "total_complete"
cluster_count_col = "cluster_count"
total_cluster_pre_col = "total_cluster_pre"
total_noise_pre_col = "total_noise_pre"
total_pruned_nn_col = "total_pruned_nn"
total_pruned_constr_col = "total_pruned_constr"
total_classified_nn_col = "total_classified_nn"
total_classified_constr_col = "total_classified_constr"
total_cluster_post_col = "total_cluster_post"
total_noise_post_col = "total_noise_post"
select_col = "select"

rep_col = "representative"
common_col = "common"
entropy_col = "entropy"
occup_col = "occupancy"

mean_col = "mean"
max_col = "max"

vect_1_col = "vector_1"
vect_2_col = "vector_2"
atom_dist_col = "atom_dist"
hb_status_col = "hydrogen_bond"
hb_angle_1_col = "hb_angle_1"
hb_angle_2_col = "hb_angle_2"
wmhb_angle_col = "wmhb_angle"
outlier_col = "outlier"

bond_col = "bond"
angle_col = "angle"

dih_col = "dihedral"
rmsd_col = "rmsd"

index_col = "index"
p_col = "p_val"
correct_p_col = "corrected_p_val"

corr_col = "correlation"

a_col = "a_val"
b_col = "b_val"
c_col = "c_val"
d_col = "d_val"
risk_ratio_col = "risk_ratio"
low_ci_col = "lower_ci"
up_ci_col = "upper_ci"
sig_col = "significance"

bb_col_lst = [phi_col, psi_col, omega_col]

sc_col_lst = [
    chi1_col,
    chi2_col,
    altchi1_col,
    altchi2_col,
    chi3_col,
    chi4_col,
    chi5_col,
]

dih_col_lst = bb_col_lst + sc_col_lst

reformat_col_lst = dih_col_lst

dist_col_lst = [
    nn_dist_col,
    constr_dist_col,
    vect_1_col,
    vect_2_col,
    atom_dist_col,
    hb_angle_1_col,
    hb_angle_2_col,
    wmhb_angle_col,
    outlier_col,
]

sum_col_lst = [common_col, entropy_col, occup_col]

data_col_lst = reformat_col_lst + dist_col_lst + sum_col_lst

class_col_lst = [
    gene_class_col,
    nuc_class_col,
    mut_class_col,
    pharm_class_col,
    match_class_col,
    prot_class_col,
    interf_class_col,
    pocket_class_col,
]

path_col_lst = [
    core_path_col,
    rcsb_path_col,
    renum_path_col,
    rcsb_assembly_path_col,
    renum_assembly_path_col,
    sifts_path_col,
    edia_path_col,
    interf_path_col,
    pocket_path_col,
]

count_col_dict = {
    pdb_id_col: total_chain_col,
    pdb_code_col: total_entry_col,
    cf_col: total_cf_col,
}

order_col_lst = [
    id_col,
    pdb_id_col,
    pdb_code_col,
    modelid_col,
    chainid_col,
    sw1_col,
    sw2_col,
    cf_col,
    cluster_col,
    total_col,
    total_chain_col,
    total_entry_col,
    total_cf_col,
    rep_col,
    rama_col,
    rotamer_col,
    nn_dist_col,
    constr_dist_col,
    interf_id_col,
    interf_col,
    pocket_id_col,
    pocket_col,
    prot_col,
    swiss_id_col,
    uniprot_id_col,
    pfam_col,
    mut_status_col,
    mut_pos_col,
    gene_class_col,
    nuc_class_col,
    mut_class_col,
    pharm_class_col,
    match_class_col,
    prot_class_col,
    interf_class_col,
    pocket_class_col,
    loop_col,
    silh_col,
    simi_col,
    total_complete_col,
    cluster_count_col,
    total_cluster_pre_col,
    total_noise_pre_col,
    total_pruned_nn_col,
    total_pruned_constr_col,
    total_classified_nn_col,
    total_classified_constr_col,
    total_cluster_post_col,
    total_noise_post_col,
    select_col,
    nn_cutoff_col,
    constr_cutoff_col,
    space_col,
    len_col,
    range_col,
    index_col,
    p_col,
    correct_p_col,
    corr_col,
    risk_ratio_col,
    low_ci_col,
    up_ci_col,
    a_col,
    b_col,
    c_col,
    d_col,
    resid_col,
    resname_col,
    atomid_col,
    interf_area_col,
    cb_cont_col,
    atomid_cont_col,
    total_cb_cont_col,
    total_atomid_cont_col,
    interf_cont_col,
    cb_dist_col,
    iso_col,
    pocket_site_col,
    pocket_volume_col,
    pocket_score_col,
    pocket_status_col,
    pocket_type_col,
    pocket_cont_col,
    phi_col,
    psi_col,
    omega_col,
    chi1_col,
    chi2_col,
    altchi1_col,
    altchi2_col,
    chi3_col,
    chi4_col,
    chi5_col,
    edia_col,
    b_factor_col,
    vect_1_col,
    vect_2_col,
    atom_dist_col,
    hb_status_col,
    hb_angle_1_col,
    hb_angle_2_col,
    wmhb_angle_col,
    outlier_col,
    bond_col,
    angle_col,
    rmsd_col,
    common_col,
    entropy_col,
    occup_col,
    bound_lig_col,
    bound_prot_col,
    bound_prot_swiss_id_col,
    pharm_lig_site_col,
    pharm_lig_match_col,
    bound_prot_pfam_col,
    bound_prot_site_col,
    bio_lig_cont_col,
    pharm_lig_cont_col,
    bound_prot_cont_col,
    bound_prot_chainid_col,
    bound_interf_chainid_col,
    bio_lig_col,
    ion_lig_col,
    pharm_lig_col,
    chem_lig_col,
    mod_lig_col,
    mem_lig_col,
    pocket_lig_col,
    method_col,
    resolution_col,
    r_factor_col,
    seq_col,
    bb_resid_col,
    bb_seq_col,
    bb_len_col,
    sc_resid_col,
    sc_seq_col,
    sc_len_col,
    complete_col,
    len_a_col,
    len_b_col,
    len_c_col,
    ang_a_col,
    ang_b_col,
    ang_g_col,
    core_path_col,
    rcsb_path_col,
    renum_path_col,
    rcsb_assembly_path_col,
    renum_assembly_path_col,
    sifts_path_col,
    edia_path_col,
    interf_path_col,
    pocket_path_col,
]

rename_col_dict = {
    pdb_id_col: "PDB ID",
    pdb_code_col: "PDB Code",
    chainid_col: "Chain",
    cf_col: "Crystal Form",
    gene_class_col: "Gene Name",
    sw1_col: "SW1 Conformation",
    sw2_col: "SW2 Conformation",
    mut_status_col: "Mutation Status",
    nuc_class_col: "Nucleotide State",
    prot_class_col: "Bound Protein",
    pharm_class_col: "Inhibitor Site",
    match_class_col: "Inhibitor Chemistry",
    pocket_class_col: "Pocket Site",
    interf_class_col: "Homodimer Status",
    bound_prot_col: "Bound Protein Name",
    bound_prot_chainid_col: "Bound Protein Chain",
    bio_lig_col: "Nucleotide",
    ion_lig_col: "Ion",
    pharm_lig_col: "Drug",
    chem_lig_col: "Chemical",
    mod_lig_col: "Modification",
    mem_lig_col: "Membrane",
    method_col: "Experiment Method",
    resolution_col: "Resolution",
    r_factor_col: "R-Factor",
    len_col: "Sequence Length",
    interf_area_col: "Interface Area",
    pocket_site_col: "Pocket Volume",
    pocket_volume_col: "Pocket Site",
    pocket_score_col: "Pocket Score",
    phi_col: "Phi",
    psi_col: "Psi",
    omega_col: "Omega",
    chi1_col: "Chi1",
    chi2_col: "Chi2",
    altchi1_col: "AltChi1",
    altchi2_col: "AltChi2",
    chi3_col: "Chi3",
    chi4_col: "Chi4",
    chi5_col: "Chi5",
}


def get_dist_col(x_resid, y_resid, x_atomid=None, y_atomid=None, ext=None):

    if ext is None:
        ext = atom_dist_col

    dist_col = str(x_resid)

    if str(x_atomid) != "None":
        dist_col += f"({x_atomid})"

    dist_col += ":"
    dist_col += str(y_resid)

    if str(y_atomid) != "None":
        dist_col += f"({y_atomid})"

    if ext is not None:
        dist_col += f"_{ext}"

    return dist_col



# -*- coding: utf-8 -*-
"""
MIT License

Copyright (c) 2022 Mitchell Isaac Parker

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in all
copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
SOFTWARE.
"""

import pandas as pd
import numpy as np
from tqdm import tqdm

from ..functions import *


def get_index_pocket(
    df,
    index,
    pocket_dict,
    min_volume=None,
    min_score=None,
    search_cont_lst=None,
    search_max_dist=0.7,
    coord_path_col=None,
    use_simpson=False,
):

    if coord_path_col is None:
        coord_path_col = core_path_col

    if search_cont_lst is not None:
        if not any(isinstance(x, list) for x in search_cont_lst):
            search_cont_lst = [search_cont_lst]
            for i, search_cont in enumerate(search_cont_lst):
                search_cont_lst[i] = [str(x) for x in search_cont]

    index_df = get_df_at_index(df, index)

    coord_path = index_df.at[index, coord_path_col]
    coord_path = coord_path.replace(".cif", ".pdb")

    pocket_df = pd.DataFrame()

    if coord_path in list(pocket_dict.keys()):
        for pocket in list(pocket_dict[coord_path].keys()):

            add_pocket = True

            if min_volume is not None:
                if (
                    float(pocket_dict[coord_path][pocket][pocket_volume_col])
                    < min_volume
                ):
                    add_pocket = False

            if min_score is not None:
                if float(pocket_dict[coord_path][pocket][pocket_score_col]) < min_score:
                    add_pocket = False

            if search_cont_lst is not None:

                pocket_cont = str_to_lst(
                    pocket_dict[coord_path][pocket][pocket_cont_col]
                )

                pocket_dist_lst = list()
                for search_cont in search_cont_lst:
                    if use_simpson:
                        pocket_dist = calc_simpson(
                            pocket_cont,
                            search_cont,
                            return_dist=True,
                        )

                    else:
                        pocket_dist = calc_jaccard(
                            pocket_cont,
                            search_cont,
                            return_dist=True,
                        )
                    pocket_dist_lst.append(pocket_dist)

                if np.mean(pocket_dist) > search_max_dist:
                    add_pocket = False

            if add_pocket:
                temp_df = index_df.copy(deep=True)

                for col in list(pocket_dict[coord_path][pocket].keys()):
                    temp_df.at[index, col] = pocket_dict[coord_path][pocket][col]

                temp_df.at[index, pocket_col] = pocket

                pocket_df = pd.concat([pocket_df, temp_df], sort=False)

    return pocket_df


def build_pocket_table(
    df,
    pocket_dict,
    pocket_table_path=None,
    min_volume=None,
    min_score=None,
    search_cont_lst=None,
    search_max_dist=0.7,
    use_simpson=False,
    coord_path_col=None,
):

    if coord_path_col is None:
        coord_path_col = core_path_col

    pocket_df = pd.DataFrame()

    for index in tqdm(
        list(df.index.values), desc="Building pocket table", position=0, leave=True
    ):

        pocket_df = pd.concat(
            [
                pocket_df,
                get_index_pocket(
                    df,
                    index,
                    pocket_dict,
                    min_volume=min_volume,
                    min_score=min_score,
                    search_cont_lst=search_cont_lst,
                    search_max_dist=search_max_dist,
                    use_simpson=use_simpson,
                    coord_path_col=coord_path_col,
                ),
            ],
            sort=False,
        )

    pocket_df = pocket_df.reset_index(drop=True)

    pocket_df[pocket_col] = pocket_df[pocket_col].map(str)
    pocket_df = fix_col(pocket_df, pocket_col)

    pocket_df[pocket_id_col] = pocket_df[pdb_id_col] + pocket_df[pocket_col]

    if pocket_table_path is not None:
        save_table(pocket_table_path, pocket_df)
    else:
        return pocket_df

    print("Built pocket table!")
